@@ -23,12 +23,16 @@
 
 #include <config.h>
 
+#include "autoptr.h"
+
 #include <xapian/error.h>
-#include <xapian/unicode.h>
-#include "omdebug.h"
+#include <xapian/positioniterator.h>
 #include <xapian/postingiterator.h>
 #include <xapian/termiterator.h>
-#include <xapian/positioniterator.h>
+#include <xapian/unicode.h>
+
+#include "omassert.h"
+#include "omdebug.h"
 #include "../backends/multi/multi_postlist.h"
 #include "../backends/multi/multi_termlist.h"
 #include "alltermslist.h"
@@ -36,6 +40,7 @@
 #include "database.h"
 #include "editdistance.h"
 #include "ortermlist.h"
+#include "noreturn.h"
 
 #include <stdlib.h> // For abs().
 
@@ -112,6 +117,11 @@ Database::postlist_begin(const string &tname) const
     // Don't bother checking that the term exists first.  If it does, we
     // just end up doing more work, and if it doesn't, we save very little
     // work.
+
+    // Handle the common case of a single database specially.
+    if (internal.size() == 1)
+	RETURN(PostingIterator(internal[0]->open_post_list(tname)));
+
     vector<LeafPostList *> pls;
     try {
 	vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
@@ -122,7 +132,7 @@ Database::postlist_begin(const string &tname) const
 	Assert(pls.begin() != pls.end());
     } catch (...) {
 	vector<LeafPostList *>::iterator i;
-	for (i = pls.begin(); i != pls.end(); i++) {
+	for (i = pls.begin(); i != pls.end(); ++i) {
 	    delete *i;
 	    *i = 0;
 	}
@@ -168,7 +178,7 @@ Database::allterms_begin(const std::string & prefix) const
 
     if (internal.size() == 1)
 	RETURN(TermIterator(internal[0]->open_allterms(prefix)));
- 
+
     vector<TermList *> lists;
 
     vector<Xapian::Internal::RefCntPtr<Database::Internal> >::const_iterator i;
@@ -492,6 +502,15 @@ Database::synonym_keys_begin(const std::string &prefix) const
     RETURN(TermIterator(merger.release()));
 }
 
+string
+Database::get_metadata(const string & key) const
+{
+    DEBUGAPICALL(string, "Database::get_metadata", key);
+    if (key.empty())
+	throw InvalidArgumentError("Empty metadata keys are invalid");
+    RETURN(internal[0]->get_metadata(key));
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 WritableDatabase::WritableDatabase() : Database()
@@ -524,10 +543,17 @@ WritableDatabase::~WritableDatabase()
     DEBUGAPICALL(void, "WritableDatabase::~WritableDatabase", "");
 }
 
+XAPIAN_NORETURN(static void only_one_subdatabase_allowed());
+static void only_one_subdatabase_allowed()
+{
+    throw Xapian::InvalidOperationError("WritableDatabase needs exactly one subdatabase");
+}
+
 void
 WritableDatabase::flush()
 {
     DEBUGAPICALL(void, "WritableDatabase::flush", "");
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->flush();
 }
 
@@ -535,6 +561,7 @@ void
 WritableDatabase::begin_transaction(bool flushed)
 {
     DEBUGAPICALL(void, "WritableDatabase::begin_transaction", "");
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->begin_transaction(flushed);
 }
 
@@ -542,6 +569,7 @@ void
 WritableDatabase::commit_transaction()
 {
     DEBUGAPICALL(void, "WritableDatabase::commit_transaction", "");
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->commit_transaction();
 }
 
@@ -549,6 +577,7 @@ void
 WritableDatabase::cancel_transaction()
 {
     DEBUGAPICALL(void, "WritableDatabase::cancel_transaction", "");
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->cancel_transaction();
 }
 
@@ -557,6 +586,7 @@ Xapian::docid
 WritableDatabase::add_document(const Document & document)
 {
     DEBUGAPICALL(Xapian::docid, "WritableDatabase::add_document", document);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     RETURN(internal[0]->add_document(document));
 }
 
@@ -564,6 +594,7 @@ void
 WritableDatabase::delete_document(Xapian::docid did)
 {
     DEBUGAPICALL(void, "WritableDatabase::delete_document", did);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (did == 0) throw InvalidArgumentError("Document ID 0 is invalid");
     internal[0]->delete_document(did);
 }
@@ -572,6 +603,7 @@ void
 WritableDatabase::delete_document(const std::string & unique_term)
 {
     DEBUGAPICALL(void, "WritableDatabase::delete_document", unique_term);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
     internal[0]->delete_document(unique_term);
@@ -582,6 +614,7 @@ WritableDatabase::replace_document(Xapian::docid did, const Document & document)
 {
     DEBUGAPICALL(void, "WritableDatabase::replace_document",
 		 did << ", " << document);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (did == 0) throw Xapian::InvalidArgumentError("Document ID 0 is invalid");
     internal[0]->replace_document(did, document);
 }
@@ -592,6 +625,7 @@ WritableDatabase::replace_document(const std::string & unique_term,
 {
     DEBUGAPICALL(Xapian::docid, "WritableDatabase::replace_document",
 		 unique_term << ", " << document);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     if (unique_term.empty())
 	throw InvalidArgumentError("Empty termnames are invalid");
     RETURN(internal[0]->replace_document(unique_term, document));
@@ -603,6 +637,7 @@ WritableDatabase::add_spelling(const std::string & word,
 {
     DEBUGAPICALL(void, "WritableDatabase::add_spelling",
 		 word << ", " << freqinc);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->add_spelling(word, freqinc);
 }
 
@@ -612,6 +647,7 @@ WritableDatabase::remove_spelling(const std::string & word,
 {
     DEBUGAPICALL(void, "WritableDatabase::remove_spelling",
 		 word << ", " << freqdec);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->remove_spelling(word, freqdec);
 }
 
@@ -621,6 +657,7 @@ WritableDatabase::add_synonym(const std::string & term,
 {
     DEBUGAPICALL(void, "WritableDatabase::add_synonym",
 		 term << ", " << synonym);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->add_synonym(term, synonym);
 }
 
@@ -630,6 +667,7 @@ WritableDatabase::remove_synonym(const std::string & term,
 {
     DEBUGAPICALL(void, "WritableDatabase::remove_synonym",
 		 term << ", " << synonym);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->remove_synonym(term, synonym);
 }
 
@@ -637,9 +675,20 @@ void
 WritableDatabase::clear_synonyms(const std::string & term) const
 {
     DEBUGAPICALL(void, "WritableDatabase::clear_synonyms", term);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
     internal[0]->clear_synonyms(term);
 }
 
+void
+WritableDatabase::set_metadata(const string & key, const string & value)
+{
+    DEBUGAPICALL(void, "WritableDatabase::set_metadata", key << ", " << value);
+    if (internal.size() != 1) only_one_subdatabase_allowed();
+    if (key.empty())
+	throw InvalidArgumentError("Empty metadata keys are invalid");
+    internal[0]->set_metadata(key, value);
+}
+
 string
 WritableDatabase::get_description() const
 {
