@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
- * Copyright 2006 Lemur Consulting Ltd
+ * Copyright 2006,2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 #include "omqueryinternal.h"
 #include "utils.h"
 #include "serialise.h"
+#include "serialise-double.h"
 
 #include <xapian/error.h>
 #include <xapian/enquire.h>
@@ -40,6 +41,7 @@
 #include <algorithm>
 #include <math.h>
 #include <limits.h>
+#include <cfloat>
 
 using namespace std;
 
@@ -58,6 +60,8 @@ get_min_subqs(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_ELITE_SET:
 	case Xapian::Query::OP_VALUE_RANGE:
 	    return 0;
+	case Xapian::Query::OP_SCALE_WEIGHT:
+	    return 1;
 	case Xapian::Query::OP_FILTER:
 	case Xapian::Query::OP_AND_MAYBE:
 	case Xapian::Query::OP_AND_NOT:
@@ -75,6 +79,8 @@ get_max_subqs(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::Internal::OP_LEAF:
 	case Xapian::Query::OP_VALUE_RANGE:
 	    return 0;
+	case Xapian::Query::OP_SCALE_WEIGHT:
+	    return 1;
 	case Xapian::Query::OP_FILTER:
 	case Xapian::Query::OP_AND_MAYBE:
 	case Xapian::Query::OP_AND_NOT:
@@ -177,6 +183,10 @@ Xapian::Query::Internal::serialise(Xapian::termpos & curpos) const
 		result += str_parameter;
 		result += om_tostring(parameter);
 		break;
+	    case Xapian::Query::OP_SCALE_WEIGHT:
+		result += ".";
+		result += str_parameter; // serialise_double(get_dbl_parameter());
+		break;
 	}
     }
     return result;
@@ -202,6 +212,7 @@ Xapian::Query::Internal::get_op_name(Xapian::Query::Internal::op_t op)
 	case Xapian::Query::OP_PHRASE:          name = "PHRASE"; break;
 	case Xapian::Query::OP_ELITE_SET:       name = "ELITE_SET"; break;
 	case Xapian::Query::OP_VALUE_RANGE:     name = "VALUE_RANGE"; break;
+	case Xapian::Query::OP_SCALE_WEIGHT:    name = "SCALE_WEIGHT"; break;
     }
     return name;
 }
@@ -236,6 +247,13 @@ Xapian::Query::Internal::get_description() const
 	return opstr;
     }
 
+    if (op == Xapian::Query::OP_SCALE_WEIGHT) {
+	opstr += om_tostring(get_dbl_parameter());
+	opstr += " * ";
+	opstr += subqs[0]->get_description();
+	return opstr;
+    }
+
     opstr = " " + get_op_name(op) + " ";
     if (op == Xapian::Query::OP_NEAR ||
 	op == Xapian::Query::OP_PHRASE ||
@@ -248,6 +266,7 @@ Xapian::Query::Internal::get_description() const
 	if (!description.empty()) description += opstr;
 	description += (**i).get_description();
     }
+
     return "(" + description + ")";
 }
 
@@ -391,6 +410,23 @@ qint_from_vector(Xapian::Query::op op,
     return qint;
 }
 
+static Xapian::Query::Internal *
+qint_from_vector(Xapian::Query::op op,
+		 const vector<Xapian::Query::Internal *> & vec,
+		 Xapian::termcount parameter,
+		 double dbl_parameter)
+{
+    Xapian::Query::Internal * qint = new Xapian::Query::Internal(op, parameter);
+    qint->set_dbl_parameter(dbl_parameter);
+    vector<Xapian::Query::Internal *>::const_iterator i;
+    for (i = vec.begin(); i != vec.end(); i++) {
+	qint->add_subquery(*i);
+	delete *i;
+    }
+    qint->end_construction();
+    return qint;
+}
+
 Xapian::Query::Internal *
 QUnserial::readcompound() {
     vector<Xapian::Query::Internal *> subqs;
@@ -451,6 +487,11 @@ QUnserial::readcompound() {
 		    return new Xapian::Query::Internal(Xapian::Query::OP_VALUE_RANGE, valno,
 						       start, stop);
 	        }
+	        case '.': {
+		    double param = unserialise_double(&p, end);
+		    return qint_from_vector(Xapian::Query::OP_SCALE_WEIGHT,
+					    subqs, 0, param);
+		}
 	        default:
 		    DEBUGLINE(UNKNOWN, "Can't parse remainder `" << p - 1 << "'");
 		    throw Xapian::InvalidArgumentError("Invalid query string");
@@ -589,6 +630,10 @@ Xapian::Query::Internal::validate_query() const
 		om_tostring(subqs.size()) + ".");
     }
 
+    if (op == OP_SCALE_WEIGHT && get_dbl_parameter() < 0) {
+	throw Xapian::InvalidArgumentError("Xapian::Query: " + get_op_name(op) + " requires a non-negative parameter.");
+    }
+
     // Check that the termname is null in a branch query, unless the op
     // is OP_VALUE_RANGE.
     Assert(is_leaf(op) || op == OP_VALUE_RANGE || tname.empty());
@@ -646,6 +691,12 @@ Xapian::Query::Internal::simplify_matchnothing()
                 subqs.erase(sq);
             }
             break;
+        case OP_SCALE_WEIGHT:
+            Assert(subqs.size() == 1);
+	    // We should have already handled OP_SCALE_WEIGHT applied to
+	    // MatchNothing in the relevant constructor.
+	    Assert(subqs[0]);
+	    break;
         case OP_LEAF:
             // Do nothing.
             break;
@@ -672,6 +723,11 @@ Xapian::Query::Internal::simplify_query()
 	    // match anything.
 	    if (tname > str_parameter) return 0;
 	    return this;
+	case OP_SCALE_WEIGHT:
+	    if (fabs(get_dbl_parameter() - 1.0) > DBL_EPSILON) return this;
+	    // If the multiplier is 1, this node doesn't actually do anything,
+	    // so we leave it to be removed.
+	    break;
 	case OP_PHRASE: case OP_NEAR:
 	    // Default to the number of subqueries.
 	    if (!parameter) parameter = subqs.size();
@@ -819,3 +875,25 @@ Xapian::Query::Internal::add_subquery(const Xapian::Query::Internal * subq)
 	subqs.push_back(new Xapian::Query::Internal(*subq));
     }
 }
+
+void
+Xapian::Query::Internal::set_dbl_parameter(double dbl_parameter_)
+{
+    // We store the double parameter encoded as a string because
+    // Xapian::Query::Internal is defined in an external API header and we want
+    // to avoid any risk of ABI breakage (we suspect it would be OK, but it's
+    // not risking).  FIXME: rework for 1.1.0
+    str_parameter = serialise_double(dbl_parameter_);
+}
+
+double
+Xapian::Query::Internal::get_dbl_parameter() const
+{
+    // We store the double parameter encoded as a string because
+    // Xapian::Query::Internal is defined in an external API header and we want
+    // to avoid any risk of ABI breakage (we suspect it would be OK, but it's
+    // not risking).  FIXME: rework for 1.1.0
+    const char * p = str_parameter.data();
+    const char * end = p + str_parameter.size();
+    return unserialise_double(&p, end);
+}
