@@ -179,17 +179,15 @@ class XAPIAN_VISIBILITY_DEFAULT FlintCursor {
 	 */
 	bool prev();
 
-	/** Find an entry, or a near match, in the table.
+	/** Position the cursor on the highest entry with key <= @a key.
 	 *
 	 *  If the exact key is found in the table, the cursor will be
 	 *  set to point to it, and the method will return true.
 	 *
 	 *  If the key is not found, the cursor will be set to point to
 	 *  the key preceding that asked for, and the method will return
-	 *  false.
-	 *
-	 *  If there is no key preceding that asked for, the cursor will
-	 *  point to a null key.
+	 *  false.  If there is no key preceding that asked for, the cursor
+	 *  will point to a null key.
 	 *
 	 *  Note:  Since the B-tree always contains a null key, which precedes
 	 *  everything, a call to FlintCursor::find_entry always results in a
@@ -205,6 +203,18 @@ class XAPIAN_VISIBILITY_DEFAULT FlintCursor {
 	 */
 	bool find_entry(const string &key);
 
+	/// Position the cursor on the highest entry with key < @a key.
+	void find_entry_lt(const string &key) {
+	    if (find_entry(key)) prev();
+	}
+
+	/** Position the cursor on the lowest entry with key >= @a key.
+	 *
+	 *  @return true if the exact key was found in the table, false
+	 *          otherwise.
+	 */
+	bool find_entry_ge(const string &key);
+
 	/** Set the cursor to be off the end of the table.
 	 */
 	void to_end() { is_after_end = true; }
@@ -218,8 +228,10 @@ class XAPIAN_VISIBILITY_DEFAULT FlintCursor {
 
 	/** Delete the current key/tag pair, leaving the cursor on the next
 	 *  entry.
+	 *
+	 *  @return false if the cursor ends up unpositioned.
 	 */
-	void del();
+	bool del();
 
 	/// Return a pointer to the FlintTable we're a cursor for.
 	FlintTable * get_table() const { return B; }
