@@ -22,11 +22,16 @@
 
 #include <config.h>
 
+#include <xapian/error.h>
+
 #include "safeerrno.h"
 #ifdef __WIN32__
 # include "msvc_posix_wrapper.h"
 #endif
 
+#include "omassert.h"
+#include "stringutils.h" // For STRINGIZE().
+
 // Define to use "dangerous" mode - in this mode we write modified btree
 // blocks back in place.  This is somewhat faster (especially once we're
 // I/O bound) but the database can't be safely searched during indexing
@@ -1023,10 +1028,10 @@ FlintTable::add(const string &key, string tag, bool already_compressed)
 
     if (key.size() > FLINT_BTREE_MAX_KEY_LEN) {
 	throw Xapian::InvalidArgumentError(
-		"Key_ too long: length was " +
+		"Key too long: length was " +
 		om_tostring(key.size()) +
 		" bytes, maximum length of a key is " +
-		STRINGIZE(FLINT_BTREE_MAX_KEY_LEN) + " bytes");
+		om_tostring(FLINT_BTREE_MAX_KEY_LEN) + " bytes");
     }
 
     form_key(key);
@@ -1559,9 +1564,6 @@ FlintTable::do_open_to_write(bool revision_supplied,
 
     writable = true;
 
-    prev_ptr = &FlintTable::prev_default;
-    next_ptr = &FlintTable::next_default;
-
     for (int j = 0; j <= level; j++) {
 	C[j].n = BLK_UNUSED;
 	C[j].p = new byte[block_size];
@@ -1819,9 +1821,6 @@ FlintTable::cancel()
 
     latest_revision_number = revision_number; // FIXME: we can end up reusing a revision if we opened a btree at an older revision, start to modify it, then cancel...
 
-    prev_ptr = &FlintTable::prev_default;
-    next_ptr = &FlintTable::next_default;
-
     for (int j = 0; j <= level; j++) {
 	C[j].n = BLK_UNUSED;
 	C[j].rewrite = false;
@@ -1865,14 +1864,6 @@ FlintTable::do_open_to_read(bool revision_supplied, flint_revision_number_t revi
 	throw Xapian::DatabaseOpeningError("Failed to open table for reading");
     }
 
-    if (sequential) {
-	prev_ptr = &FlintTable::prev_for_sequential;
-	next_ptr = &FlintTable::next_for_sequential;
-    } else {
-	prev_ptr = &FlintTable::prev_default;
-	next_ptr = &FlintTable::next_default;
-    }
-
     for (int j = 0; j <= level; j++) {
 	C[j].n = BLK_UNUSED;
 	C[j].p = new byte[block_size];
