@@ -31,6 +31,7 @@
 #include "inmemory_database.h"
 #include "inmemory_document.h"
 #include "inmemory_alltermslist.h"
+#include "utils.h"
 
 #include <string>
 #include <vector>
@@ -252,7 +253,7 @@ InMemoryTermList::at_end() const
 Xapian::termcount
 InMemoryTermList::positionlist_count() const
 {
-    throw Xapian::UnimplementedError("InMemoryTermList::positionlist_count() not implemented");
+    return db->positionlist_count(did, (*pos).tname);
 }
 
 Xapian::PositionIterator
@@ -457,6 +458,44 @@ InMemoryDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
 				valuelists[did - 1]);
 }
 
+std::string
+InMemoryDatabase::get_metadata(const std::string & key) const
+{
+    map<string, string>::const_iterator i = metadata.find(key);
+    if (i == metadata.end())
+	return string();
+    return i->second;
+}
+
+void
+InMemoryDatabase::set_metadata(const std::string & key,
+			       const std::string & value)
+{
+    if (!value.empty()) {
+	metadata[key] = value;
+    } else {
+	metadata.erase(key);
+    }
+}
+
+Xapian::termcount
+InMemoryDatabase::positionlist_count(Xapian::docid did,
+				     const string & tname) const
+{
+    if (!doc_exists(did)) {
+	return 0;
+    }
+    const InMemoryDoc &doc = termlists[did-1];
+
+    vector<InMemoryTermEntry>::const_iterator i;
+    for (i = doc.terms.begin(); i != doc.terms.end(); ++i) {
+	if (i->tname == tname) {
+	    return i->positions.size();
+	}
+    }
+    return 0;
+}
+
 PositionList * 
 InMemoryDatabase::open_position_list(Xapian::docid did,
 				     const string & tname) const
