@@ -22,8 +22,10 @@
 #include <config.h>
 #include <stdio.h>
 
+#include "omassert.h"
 #include "omdebug.h"
 #include "multi_postlist.h"
+#include "utils.h"
 
 #ifdef XAPIAN_DEBUG_PARANOID
 #include "xapian/database.h"
@@ -57,21 +59,10 @@ MultiPostList::~MultiPostList()
     postlists.clear();
 }
 
-void
-MultiPostList::set_termweight(const Xapian::Weight * wt)
-{
-    // Set in base class, so that get_maxweight() works
-    LeafPostList::set_termweight(wt);
-    std::vector<LeafPostList *>::const_iterator i;
-    for (i = postlists.begin(); i != postlists.end(); i++) {
-	(*i)->set_termweight(wt);
-    }
-}
-
 Xapian::doccount
 MultiPostList::get_termfreq() const
 {
-    if(freq_initialised) return termfreq;
+    if (freq_initialised) return termfreq;
     DEBUGLINE(DB, "Calculating multiple term frequencies");
 
     // Calculate and remember the termfreq
@@ -166,11 +157,13 @@ MultiPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
     DEBUGCALL(DB, PostList *, "MultiPostList::skip_to", did << ", " << w_min);
     Assert(!at_end());
     Xapian::docid newdoc = 0;
-    Xapian::docid offset = 1;
-    Xapian::docid realdid = (did - 1) / multiplier + 1;
+    Xapian::docid offset = 0;
+    Xapian::docid realdid = (did - 1) / multiplier + 2;
     Xapian::doccount dbnumber = (did - 1) % multiplier;
     std::vector<LeafPostList *>::iterator i;
     for (i = postlists.begin(); i != postlists.end(); i++) {	
+	if (offset == dbnumber) --realdid;
+	++offset;
 	Assert((realdid - 1) * multiplier + offset >= did);
 	Assert((realdid - 1) * multiplier + offset < did + multiplier);
 	if (!(*i)->at_end()) {
@@ -180,8 +173,6 @@ MultiPostList::skip_to(Xapian::docid did, Xapian::weight w_min)
 		if (newdoc == 0 || id < newdoc) newdoc = id;
 	    }
 	}
-	offset++;
-	if (offset == dbnumber) realdid--;
     }
     if (newdoc) {
 	currdoc = newdoc;
