@@ -121,7 +121,7 @@ QuartzDatabase::QuartzDatabase(const string &quartz_dir, int action,
 		fail = true;
 	    }
 	    if (fail) {
-		throw Xapian::DatabaseOpeningError("Cannot create directory `"
+		throw Xapian::DatabaseCreateError("Cannot create directory `"
 						   + db_dir + "'", errno);
 	    }
 	    get_database_write_lock();
