@@ -29,6 +29,7 @@
 #include "flint_table.h"
 #include "flint_types.h"
 #include "flint_utils.h"
+#include "stringutils.h"
 #include "utils.h"
 
 #include <xapian.h>
@@ -244,8 +245,16 @@ main(int argc, char **argv)
 		errors += check_table(table, opts);
 	    }
 	} else {
-	    // Just check a single Btree.
-	    errors = check_table(argv[1], opts);
+	    // Just check a single Btree.  If it ends with "." or ".DB"
+	    // already, trim that so the user can do xapian-check on
+	    // "foo", "foo.", or "foo.DB".
+	    string table_name = argv[1];
+	    if (endswith(table_name, '.'))
+		table_name.resize(table_name.size() - 1);
+	    else if (endswith(table_name, ".DB"))
+		table_name.resize(table_name.size() - 3);
+
+	    errors = check_table(table_name, opts);
 	}
 	if (errors > 0) {
 	    cout << "Total errors found: " << errors << endl;
@@ -341,6 +350,7 @@ check_table(string filename, int opts)
 	    if (!unpack_string_preserving_sort(&pos, end, term)) {
 		cout << "Error unpacking termname from key" << endl;
 		++errors;
+		cursor->next();
 		continue;
 	    }
 	    if (current_term.empty()) {
@@ -350,6 +360,7 @@ check_table(string filename, int opts)
 		    cout << "Extra bytes after key for first chunk of "
 			"posting list for term `" << term << "'" << endl;
 		    ++errors;
+		    cursor->next();
 		    continue;
 		}
 		// Unpack extra header from first chunk.
@@ -360,18 +371,21 @@ check_table(string filename, int opts)
 		    cout << "Failed to unpack termfreq for term `" << term
 			 << "'" << endl;
 		    ++errors;
+		    cursor->next();
 		    continue;
 		}
 		if (!unpack_uint(&pos, end, &collfreq)) {
 		    cout << "Failed to unpack collfreq for term `" << term
 			 << "'" << endl;
 		    ++errors;
+		    cursor->next();
 		    continue;
 		}
 		if (!unpack_uint(&pos, end, &did)) {
 		    cout << "Failed to unpack firstdid for term `" << term
 			 << "'" << endl;
 		    ++errors;
+		    cursor->next();
 		    continue;
 		} else {
 		    ++did;
@@ -393,6 +407,7 @@ check_table(string filename, int opts)
 		    if (!unpack_uint_preserving_sort(&pos, end, &did)) {
 			cout << "Failed to unpack did from key" << endl;
 			++errors;
+			cursor->next();
 			continue;
 		    }
 		    if (did <= lastdid) {
@@ -410,12 +425,14 @@ check_table(string filename, int opts)
 	    if (!unpack_bool(&pos, end, &is_last_chunk)) {
 		cout << "Failed to unpack last chunk flag" << endl;
 		++errors;
+		cursor->next();
 		continue;
 	    }
 	    // Read what the final document ID in this chunk is.
 	    if (!unpack_uint(&pos, end, &lastdid)) {
 		cout << "Failed to unpack increase to last" << endl;
 		++errors;
+		cursor->next();
 		continue;
 	    }
 	    ++lastdid;
@@ -463,7 +480,10 @@ check_table(string filename, int opts)
 		    ++errors;
 		}
 	    }
-	    if (bad) continue;
+	    if (bad) {
+		cursor->next();
+		continue;
+	    }
 	    if (is_last_chunk) {
 		if (tf != termfreq) {
 		    cout << "termfreq " << termfreq << " != # of entries "
@@ -534,8 +554,13 @@ check_table(string filename, int opts)
 	    pos = cursor->current_tag.data();
 	    end = pos + cursor->current_tag.size();
 
+	    if (pos == end) {
+		// Empty termlist.
+		cursor->next();
+		continue;
+	    }
+
 	    Xapian::termcount doclen, termlist_size;
-	    bool has_termfreqs;
 
 	    // Read doclen
 	    if (!unpack_uint(&pos, end, &doclen)) {
@@ -545,6 +570,7 @@ check_table(string filename, int opts)
 		    cout << "Unexpected end of data when reading doclen" << endl;
 		}
 		++errors;
+		cursor->next();
 		continue;
 	    }
 
@@ -556,20 +582,13 @@ check_table(string filename, int opts)
 		    cout << "Unexpected end of data when reading termlist_size" << endl;
 		}
 		++errors;
+		cursor->next();
 		continue;
 	    }
 
-	    // Read has_termfreqs
-	    if (!unpack_bool(&pos, end, &has_termfreqs)) {
-		cout << "Unexpected end of data when reading termlist" << endl;
-		++errors;
-		continue;
-	    }
-	    if (has_termfreqs) {
-		cout << "has_termfreqs is true, but Xapian never sets it!" << endl;
-		++errors;
-		continue;
-	    }
+	    // See comment in FlintTermListTable::set_termlist() in
+	    // flint_termlisttable.cc for an explanation of this!
+	    if (pos != end && *pos == '0') ++pos;
 
 	    Xapian::termcount actual_doclen = 0, actual_termlist_size = 0;
 	    string current_tname;
@@ -599,7 +618,7 @@ check_table(string filename, int opts)
 		    // Read wdf
 		    if (!unpack_uint(&pos, end, &current_wdf)) {
 			if (pos == 0) {
-			    cout << "Unexpected end of data when reading termlist" << endl;
+			    cout << "Unexpected end of data when reading termlist current_wdf" << endl;
 			} else {
 			    cout << "Size of wdf out of range, in termlist" << endl;
 			}
@@ -609,13 +628,13 @@ check_table(string filename, int opts)
 		    }
 		}
 
-		// Don't bother with the (has_termfreqs == true) case since
-		// we never generate that.
-
 		++actual_termlist_size;
 		actual_doclen += current_wdf;
 	    }
-	    if (bad) continue;
+	    if (bad) {
+		cursor->next();
+		continue;
+	    }
 
 	    if (termlist_size != actual_termlist_size) {
 		cout << "termlist_size != # of entries in termlist" << endl;
@@ -719,6 +738,7 @@ check_table(string filename, int opts)
 	    if (!unpack_uint(&pos, end, &pos_last)) {
 		cout << tablename << " table: Position list data corrupt" << endl;
 		++errors;
+		cursor->next();
 		continue;
 	    }
 	    if (pos == end) {
@@ -741,6 +761,7 @@ check_table(string filename, int opts)
 		    if (pos <= lastpos) {
 			cout << tablename << " table: Positions not strictly monotonically increasing" << endl;
 			++errors;
+			cursor->next();
 			continue;
 		    }
 		    lastpos = pos;
