@@ -26,6 +26,7 @@
 #include <xapian/database.h>
 #include <xapian/document.h>
 #include <xapian/enquire.h>
+#include <xapian/sorter.h>
 #include <xapian/query.h>
 #include <algorithm>
 #include <math.h>
@@ -44,7 +45,7 @@ class ErrorHandler;
 class TermIterator;
 
 namespace Internal {
- 
+
 /** An item in the ESet.
  *  This item contains the termname, and the weight calculated for
  *  the document.
@@ -52,6 +53,12 @@ namespace Internal {
 class ESetItem {
     public:
 	ESetItem(Xapian::weight wt_, string tname_) : wt(wt_), tname(tname_) { }
+
+	void swap(ESetItem & o) {
+	    std::swap(wt, o.wt);
+	    std::swap(tname, o.tname);
+	}
+
 	/// Weight calculated.
 	Xapian::weight wt;
 	/// Term suggested.
@@ -80,6 +87,14 @@ class MSetItem {
 		: wt(wt_), did(did_), collapse_key(key_),
 		  collapse_count(collapse_count_) {}
 
+	void swap(MSetItem & o) {
+	    std::swap(wt, o.wt);
+	    std::swap(did, o.did);
+	    std::swap(collapse_key, o.collapse_key);
+	    std::swap(collapse_count, o.collapse_count);
+	    std::swap(sort_key, o.sort_key);
+	}
+
 	/** Weight calculated. */
 	Xapian::weight wt;
 
@@ -154,12 +169,12 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 	sort_setting sort_by;
 	bool sort_value_forward;
 
+	Sorter * sorter;
+
 	/** The error handler, if set.  (0 if not set).
 	 */
 	ErrorHandler * errorhandler;
 
-	map<string, const MatchDecider *> mdecider_map;
-
 	mutable Weight * weight; // mutable so get_mset can set default
 
 	Internal(const Xapian::Database &databases, ErrorHandler * errorhandler_);
@@ -192,10 +207,11 @@ class Enquire::Internal : public Xapian::Internal::RefCntBase {
 };
 
 class MSet::Internal : public Xapian::Internal::RefCntBase {
-    private:
+    public:
 	/// Factor to multiply weights by to convert them to percentages.
 	double percent_factor;
 
+    private:
 	/** The set of documents which have been requested but not yet
 	 *  collected.
 	 */
@@ -220,6 +236,9 @@ class MSet::Internal : public Xapian::Internal::RefCntBase {
 	 *  given query term.
 	 */
 	struct TermFreqAndWeight {
+	    TermFreqAndWeight() { }
+	    TermFreqAndWeight(Xapian::doccount tf, Xapian::weight wt)
+		: termfreq(tf), termweight(wt) { }
 	    Xapian::doccount termfreq;
 	    Xapian::weight termweight;
 	};
