@@ -36,12 +36,17 @@
 // 30.1: Pass the prefix parameter for MSG_ALLTERMS, and use it.
 // 30.2: New REPLY_DELETEDOCUMENT returns MSG_DONE to allow exceptions.
 // 30.3: New MSG_GETMSET which passes check_at_least parameter.
+// 30.4: New query operator OP_SCALE_WEIGHT.
+// 30.5: New MSG_GETMSET which expects MSet's percent_factor to be returned.
 #define XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION 30
-#define XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION 3
+#define XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION 5
 
 /* When we move to version 31:
  * + Remove MSG_DELETEDOCUMENT_PRE_30_2
  * + Remove MSG_GETMSET_PRE_30_3
+ * + Remove MSG_GETMSET_PRE_30_5
+ * + Remove REPLY_RESULTS_PRE_30_5
+ * Include MSet::Internal::percent_factor in the MSet serialisation.
  */
 
 /// Message types (client -> server).
@@ -66,9 +71,10 @@ enum message_type {
     MSG_FLUSH,			// Flush
     MSG_REPLACEDOCUMENT,	// Replace Document
     MSG_REPLACEDOCUMENTTERM,	// Replace Document by term
-    MSG_GETMSET_PRE_30_3,	// Get MSet
+    MSG_GETMSET_PRE_30_3,	// Get MSet for < 30.3
     MSG_SHUTDOWN,		// Shutdown
     MSG_DELETEDOCUMENT,		// Delete Document
+    MSG_GETMSET_PRE_30_5,	// Get MSet for < 30.5
     MSG_GETMSET,		// Get MSet
     MSG_MAX
 };
@@ -85,7 +91,7 @@ enum reply_type {
     REPLY_TERMEXISTS,		// Term Exists
     REPLY_TERMFREQ,		// Get Term Frequency
     REPLY_DOCLENGTH,		// Get Doc Length
-    REPLY_RESULTS,		// Results (MSet)
+    REPLY_RESULTS_PRE_30_5,	// Results (MSet) for < 30.5
     REPLY_STATS,		// Stats
     REPLY_TERMLIST,		// Get Termlist
     REPLY_POSITIONLIST,		// Get PositionList
@@ -94,6 +100,7 @@ enum reply_type {
     REPLY_UPDATE,		// Get Updated DocCount and AvLength
     REPLY_VALUE,		// Document Value
     REPLY_ADDDOCUMENT,		// Add Document
+    REPLY_RESULTS,		// Results (MSet)
     REPLY_MAX
 };
 
