@@ -45,9 +45,9 @@
 // Searching
 #include <xapian/enquire.h>
 #include <xapian/expanddecider.h>
-#include <xapian/matchspy.h>
 #include <xapian/query.h>
 #include <xapian/queryparser.h>
+#include <xapian/sorter.h>
 
 // Stemming
 #include <xapian/stem.h>
