@@ -106,10 +106,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 */
 	void reopen();
 
-	/** Introspection method.
-	 *
-	 *  @return A string describing this object.
-	 */
+	/// Return a string describing this object.
 	virtual std::string get_description() const;
 
 	/** An iterator pointing to the start of the postlist
@@ -204,9 +201,9 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/** Return the total number of occurrences of the given term.
 	 *
-	 *  This is the sum of the number of ocurrences of the term in each
-	 *  document it indexes: ie, the sum of the within document frequencies
-	 *  of the term.
+	 *  This is the sum of the number of occurrences of the term in each
+	 *  document it indexes: i.e., the sum of the within document
+	 *  frequencies of the term.
 	 *
 	 *  @param tname  The term whose collection frequency is being
 	 *  requested.
@@ -283,6 +280,36 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	Xapian::TermIterator synonym_keys_end(const std::string & = "") const {
 	    return Xapian::TermIterator(NULL);
 	}
+
+	/** Get the user-specified metadata associated with a given key.
+	 *
+	 *  User-specified metadata allows you to store arbitrary information
+	 *  in the form of (key,tag) pairs.  See @a
+	 *  WritableDatabase::set_metadata() for more information.
+	 *
+	 *  When invoked on a Xapian::Database object representing multiple
+	 *  databases, currently only the metadata for the first is considered
+	 *  but this behaviour may change in the future.
+	 *
+	 *  If there is no piece of metadata associated with the specified
+	 *  key, an empty string is returned (this applies even for backends
+	 *  which don't support metadata).
+	 *
+	 *  Empty keys are not valid, and specifying one will cause an
+	 *  exception.
+	 *
+	 *  @param key The key of the metadata item to access.
+	 *
+	 *  @return    The retrieved metadata item's value.
+	 *
+	 *  @exception Xapian::InvalidArgumentError will be thrown if the
+	 *	       key supplied is empty.
+	 *
+	 *  @exception Xapian::UnimplementedError will be thrown if the
+	 *	       database backend in use doesn't support user-specified
+	 *	       metadata.
+	 */
+	std::string get_metadata(const std::string & key) const;
 };
 
 /** This class provides read/write access to a database.
@@ -645,10 +672,49 @@ class XAPIAN_VISIBILITY_DEFAULT WritableDatabase : public Database {
 	 */
 	void clear_synonyms(const std::string & term) const;
 
-	/** Introspection method.
+	/** Set the user-specified metadata associated with a given key.
 	 *
-	 *  @return A string describing this object.
+	 *  This method sets the metadata value associated with a given key.
+	 *  If there is already a metadata value stored in the database with
+	 *  the same key, the old value is replaced.  If you want to delete an
+	 *  existing item of metadata, just set its value to the empty string.
+	 *
+	 *  User-specified metadata allows you to store arbitrary information
+	 *  in the form of (key,tag) pairs.
+	 *
+	 *  There's no hard limit on the number of metadata items, or the size
+	 *  of the metadata values.  Metadata keys have a limited length, which
+	 *  depends on the backend.  We recommend limiting them to 200 bytes.
+	 *  Empty keys are not valid, and specifying one will cause an
+	 *  exception.
+	 *
+	 *  Metadata modifications are committed to disk in the same way as
+	 *  modifications to the documents in the database are: i.e.,
+	 *  modifications are atomic, and won't be committed to disk
+	 *  immediately (see flush() for more details).  This allows metadata
+	 *  to be used to link databases with versioned external resources
+	 *  by storing the appropriate version number in a metadata item.
+	 *
+	 *  You can also use the metadata to store arbitrary extra information
+	 *  associated with terms, documents, or postings by encoding the
+	 *  termname and/or document id into the metadata key.
+	 *
+	 *  @param key   The key of the metadata item to set.
+	 *
+	 *  @param value The value of the metadata item to set.
+	 *
+	 *  @exception Xapian::DatabaseError will be thrown if a problem occurs
+	 *             while writing to the database.
+	 *
+	 *  @exception Xapian::DatabaseCorruptError will be thrown if the
+	 *             database is in a corrupt state.
+	 *
+	 *  @exception Xapian::InvalidArgumentError will be thrown if the
+	 *             key supplied is empty.
 	 */
+	void set_metadata(const std::string & key, const std::string & value);
+
+	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
