@@ -28,6 +28,7 @@
 
 #include <xapian/base.h>
 #include <xapian/deprecated.h>
+#include <xapian/sorter.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
 #include <xapian/visibility.h>
@@ -229,9 +230,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	typedef Xapian::doccount size_type;
 	//@}
 
-	/** Returns a string representing the MSet.
-	 *  Introspection method.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
@@ -359,9 +358,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
 	 */
 	Xapian::percent get_percent() const;
 
-	/** Returns a string describing this object.
-	 *  Introspection method.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 
 	/// Allow use as an STL iterator
@@ -438,10 +435,7 @@ class XAPIAN_VISIBILITY_DEFAULT ESet {
 	/** This returns the term at position i in this E-Set.  */
 	ESetIterator operator[](Xapian::termcount i) const;
 
-	/** Introspection method.
-	 *
-	 *  @return  A string representing this ESet.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
@@ -510,9 +504,7 @@ class XAPIAN_VISIBILITY_DEFAULT ESetIterator {
 	/// Get the weight of the term at the current position
 	Xapian::weight get_weight() const;
 
-	/** Returns a string describing this object.
-	 *  Introspection method.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 
 	/// Allow use as an STL iterator
@@ -583,10 +575,7 @@ class XAPIAN_VISIBILITY_DEFAULT RSet {
 	/// Test if a given document in the relevance set.
 	bool contains(const Xapian::MSetIterator & i) const { return contains(*i); }
 
-	/** Introspection method.
-	 *
-	 *  @return  A string representing this RSet.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
@@ -595,6 +584,9 @@ class XAPIAN_VISIBILITY_DEFAULT RSet {
 class XAPIAN_VISIBILITY_DEFAULT MatchDecider {
     public:
 	/** Decide whether we want this document to be in the MSet.
+	 *
+	 *  Return true if the document is acceptable, or false if the document
+	 *  should be excluded from the MSet.
 	 */
 	virtual bool operator()(const Xapian::Document &doc) const = 0;
 
@@ -613,14 +605,13 @@ class XAPIAN_VISIBILITY_DEFAULT MatchDecider {
  *  argument is supplied, for example, an unknown database type.
  */
 class XAPIAN_VISIBILITY_DEFAULT Enquire {
-    private:
-	/// Copies are not allowed.
-	Enquire(const Enquire &);
-
-	/// Assignment is not allowed.
-	void operator=(const Enquire &);
-
     public:
+	/// Copying is allowed (and is cheap).
+	Enquire(const Enquire & other);
+
+	/// Assignment is allowed (and is cheap).
+	void operator=(const Enquire & other);
+
 	class Internal;
 	/// @internal Reference counted internals.
 	Xapian::Internal::RefCntPtr<Internal> internal;
@@ -778,6 +769,16 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
          */
 	void set_sort_by_value(Xapian::valueno sort_key, bool ascending = true);
 
+	/** Set the sorting to be by key generated from values only.
+	 *
+	 * @param sorter    The functor to use for generating keys.
+	 *
+	 * @param ascending  If true, documents values which sort higher by
+	 *		 string compare are better.  If false, the sort order
+	 *		 is reversed.  (default true)
+         */
+	void set_sort_by_key(Xapian::Sorter * sorter, bool ascending = true);
+
 	/** Set the sorting to be by value, then by relevance for documents
 	 *  with the same value.
 	 *
@@ -794,6 +795,18 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	void set_sort_by_value_then_relevance(Xapian::valueno sort_key,
 					      bool ascending = true);
 
+	/** Set the sorting to be by keys generated from values, then by
+	 *  relevance for documents with identical keys.
+	 *
+	 * @param sorter    The functor to use for generating keys.
+	 *
+	 * @param ascending  If true, keys which sort higher by
+	 *		 string compare are better.  If false, the sort order
+	 *		 is reversed.  (default true)
+	 */
+	void set_sort_by_key_then_relevance(Xapian::Sorter * sorter,
+					    bool ascending = true);
+
 	/** Set the sorting to be by relevance then value.
 	 *
 	 *  NB sorting of values uses a string comparison, so you'll need to
@@ -816,6 +829,25 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	void set_sort_by_relevance_then_value(Xapian::valueno sort_key,
 					      bool ascending = true);
 
+	/** Set the sorting to be by relevance, then by keys generated from
+	 *  values.
+	 *
+	 *  Note that with the default BM25 weighting scheme parameters,
+	 *  non-identical documents will rarely have the same weight, so
+	 *  this setting will give very similar results to
+	 *  set_sort_by_relevance().  It becomes more useful with particular
+	 *  BM25 parameter settings (e.g. BM25Weight(1,0,1,0,0)) or custom
+	 *  weighting schemes.
+	 *
+	 * @param sorter    The functor to use for generating keys.
+	 *
+	 * @param ascending  If true, keys which sort higher by
+	 *		 string compare are better.  If false, the sort order
+	 *		 is reversed.  (default true)
+	 */
+	void set_sort_by_relevance_then_key(Xapian::Sorter * sorter,
+					    bool ascending = true);
+
 	/** Get (a portion of) the match set for the current query.
 	 *
 	 *  @param first     the first item in the result set to return.
@@ -989,23 +1021,29 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	}
 
 	/** Register a MatchDecider.
+	 *
+	 * This is used to associate a name with a matchdecider.
+	 *
+	 * @deprecated This method is deprecated.  It was added long ago with
+	 * the intention that it would allow the remote backend to support
+	 * use of MatchDecider objects, but there's a better approach.
 	 *
 	 * @param name		The name to register this matchdecider as.
 	 * @param mdecider	The matchdecider.  If omitted, then remove
 	 *			any matchdecider registered with this name.
 	 */
+	XAPIAN_DEPRECATED(
 	void register_match_decider(const std::string &name,
-				    const MatchDecider *mdecider = NULL);
+				    const MatchDecider *mdecider = NULL));
 
-	/** Introspection method.
-	 *  @return  A string representing the enquire object.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 };
 
 }
 
 class RemoteServer;
+class ScaleWeight;
 
 namespace Xapian {
 
@@ -1013,6 +1051,7 @@ namespace Xapian {
 class XAPIAN_VISIBILITY_DEFAULT Weight {
     friend class Enquire; // So Enquire can clone us
     friend class ::RemoteServer; // So RemoteServer can clone us - FIXME
+    friend class ::ScaleWeight;
     public:
 	class Internal;
     protected:
@@ -1032,12 +1071,13 @@ class XAPIAN_VISIBILITY_DEFAULT Weight {
 	virtual Weight * clone() const = 0;
 
     protected:
-	const Internal * internal; // Weight::Internal == StatsSource
+	const Internal * internal; // Weight::Internal == Stats
 	Xapian::doclength querysize;
 	Xapian::termcount wqf;
 	std::string tname;
 
     public:
+	// FIXME:1.1: initialise internal to NULL here
 	Weight() { }
 	virtual ~Weight();
 
