@@ -4,7 +4,7 @@
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
  * Copyright 2003,2004,2005,2006,2007 Olly Betts
- * Copyright 2006 Lemur Consulting Ltd
+ * Copyright 2006,2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include <vector>
 
 #include <xapian/base.h>
+#include <xapian/deprecated.h>
 #include <xapian/types.h>
 #include <xapian/termiterator.h>
 #include <xapian/visibility.h>
@@ -36,8 +37,9 @@
 // FIXME: sort this out so we avoid exposing Xapian::Query::Internal
 // - we need to at present so that the Xapian::Query's template ctors
 // compile.
-class MultiMatch;
 class LocalSubMatch;
+class MultiMatch;
+class QueryOptimiser;
 struct SortPosName;
 
 namespace Xapian {
@@ -96,10 +98,20 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	    /** Filter by a range test on a document value. */
 	    OP_VALUE_RANGE,
 
+	    /** Scale the weight of a subquery by the specified factor.
+	     *
+	     *  A factor of 0 means this subquery will contribute no weight to
+	     *  the query - it will act as a purely boolean subquery.
+	     *
+	     *  If the factor is negative, Xapian::InvalidArgumentError will
+	     *  be thrown.
+	     */
+	    OP_SCALE_WEIGHT,
+
 	    /** Select an elite set from the subqueries, and perform
 	     *  a query with these combined as an OR query.
 	     */
-	    OP_ELITE_SET = 10
+	    OP_ELITE_SET
 	} op;
 
 	/** Copy constructor. */
@@ -151,8 +163,18 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	Query(Query::op op_, Iterator qbegin, Iterator qend,
 	      Xapian::termcount parameter = 0);
 
-	/** Apply the specified operator to a single Xapian::Query object. */
-	Query(Query::op op_, Xapian::Query q);
+	/** Apply the specified operator to a single Xapian::Query object.
+	 *
+	 *  @deprecated This method is deprecated because it isn't useful,
+	 *  since none of the current query operators can be usefully
+	 *  applied to a single subquery with a parameter value.
+	 */
+	XAPIAN_DEPRECATED(Query(Query::op op_, Xapian::Query q));
+
+	/** Apply the specified operator to a single Xapian::Query object, with
+	 *  a double parameter.
+	 */
+	Query(Query::op op_, Xapian::Query q, double parameter);
 
 	/** Construct a value range query on a document value.
 	 *
@@ -201,9 +223,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 */
 	bool empty() const;
 
-	/** Returns a string representing the query.
-	 *  Introspection method.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 
     private:
@@ -237,9 +257,11 @@ Query::Query(Query::op op_, Iterator qbegin, Iterator qend, termcount parameter)
 
 /// @internal Internal class, implementing most of Xapian::Query.
 class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCntBase {
-    friend class ::MultiMatch;
     friend class ::LocalSubMatch;
+    friend class ::MultiMatch;
+    friend class ::QueryOptimiser;
     friend struct ::SortPosName;
+    friend class Query;
     public:
         static const int OP_LEAF = -1;
 
@@ -251,7 +273,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 
     private:
 	/// Operation to be performed at this node
-	op_t op;
+	Xapian::Query::Internal::op_t op;
 
 	/// Sub queries on which to perform operation
 	subquery_list subqs;
@@ -360,6 +382,10 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	 */
 	void add_subquery(const Query::Internal * subq);
 
+	void set_dbl_parameter(double dbl_parameter_);
+
+	double get_dbl_parameter() const;
+
 	/** Finish off the construction.
 	 */
 	Query::Internal * end_construction();
@@ -372,9 +398,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
             return serialise(curpos);
         }
 
-	/** Returns a string representing the query.
-	 * Introspection method.
-	 */
+	/// Return a string describing this object.
 	std::string get_description() const;
 
 	/** Get the numeric parameter used in this query.
