@@ -152,14 +152,19 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public ValueRangeProce
 };
 
 // Xapian 1.0.0 and 1.0.1 had a conceptually broken implementation of
-// NumberValueRangeProcessor which we quickly told people to avoid using.  But to keep
-// ABI compatibility, we should keep it around until the next incompatible ABI change
-// (probably 1.1.0).  So we put the new NumberValueRangeProcessor in a subnamespace and
-// then pull it into this one with "using v102::NumberValueRangeProcessor".  The old
+// NumberValueRangeProcessor which we quickly told people to avoid using.  But
+// to keep ABI compatibility, we should keep it around until the next
+// incompatible ABI change (probably 1.1.0).  So we put the new
+// NumberValueRangeProcessor in a subnamespace and then pull it into this one
+// with "using v102::NumberValueRangeProcessor".  The old
 // NumberValueRangeProcessor still exists with default visibility, but isn't
-// declared in an external or internal header, so will only be used when dynamically
-// linking with application code built against Xapian 1.0.0 or 1.0.1.
+// declared in an external or internal header, so will only be used when
+// dynamically linking with application code built against Xapian 1.0.0 or
+// 1.0.1.
+#ifndef DOXYGEN
+// Hide the v102 namespace from Doxygen as it isn't user-visible.
 namespace v102 {
+#endif
 
 /** Handle a number range.
  *
@@ -229,10 +234,12 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public ValueRangePro
     Xapian::valueno operator()(std::string &begin, std::string &end);
 };
 
+#ifndef DOXYGEN
 }
 
-#ifndef XAPIAN_NO_V102_NUMBER_VRP
+# ifndef XAPIAN_NO_V102_NUMBER_VRP
 using v102::NumberValueRangeProcessor;
+# endif
 #endif
 
 /// Build a Xapian::Query object from a user query string.
@@ -343,11 +350,14 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  This controls how the query parser will apply the stemming algorithm.
      *  The default value is STEM_NONE.  The possible values are:
      *
-     *   - STEM_NONE: performs no stemming.
-     *   - STEM_SOME: Search for stemmed forms of words which do not begin
-     *                with a capital letter, and search for unstemmed forms
-     *                of words which do begin with a capital letter.
-     *   - STEM_ALL:  Search for stemmed forms of all words.
+     *   - STEM_NONE: Don't perform any stemming.
+     *   - STEM_SOME: Search for stemmed forms of terms except for those which
+     *		      start with a capital letter, or are followed by certain
+     *		      characters (currently: (/\@<>=*[{" ), or are used with
+     *		      operators which need positional information.  Stemmed
+     *		      terms are prefixed with 'Z'.
+     *   - STEM_ALL:  Search for stemmed forms of all words (note: no 'Z'
+     *		      prefix is added).
      *
      *  Note that the stemming algorithm is only applied to words in
      *  probabilistic fields - boolean filter terms are never stemmed.
@@ -381,12 +391,35 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 
     /** Add a probabilistic term prefix.
      *
-     *  E.g. qp.add_prefix("author", "A");
+     *  For example:
      *
-     *  Allows the user to search for author:orwell which will search
-     *  for the term "Aorwel" (assuming English stemming is in use).
-     *  Multiple fields can be mapped to the same prefix (so you can
-     *  e.g. make title: and subject: aliases for each other).
+     *  @code
+     *  qp.add_prefix("author", "A");
+     *  @endcode
+     *
+     *  This allows the user to search for author:Orwell which will be
+     *  converted to a search for the term "Aorwell".
+     *
+     *  Multiple fields can be mapped to the same prefix.  For example, you
+     *  can make title: and subject: aliases for each other.
+     *
+     *  As of 1.0.4, you can call this method multiple times with the same
+     *  value of field to allow a single field to be mapped to multiple
+     *  prefixes.  Multiple terms being generated for such a field, and
+     *  combined with @c Xapian::Query::OP_OR.
+     *
+     *  If any prefixes are specified for the empty field name (i.e. you
+     *  call this method with an empty string as the first parameter)
+     *  these prefixes will be used as the default prefix.  If you do
+     *  this and also specify the @c default_prefix parameter to
+     *  @c parse_query(), then the @c default_prefix parameter will override.
+     *
+     *  If you call @c add_prefix() and @c add_boolean_prefix() for the
+     *  same value of @a field, a @c Xapian::InvalidOperationError exception
+     *  will be thrown.
+     *
+     *  In 1.0.3 and earlier, subsequent calls to this method with the same
+     *  value of @a field had no effect.
      *
      *  @param field   The user visible field name
      *  @param prefix  The term prefix to map this to
@@ -396,26 +429,45 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     /** Add a boolean term prefix allowing the user to restrict a
      *  search with a boolean filter specified in the free text query.
      *
-     *  E.g. qp.add_boolean_prefix("site", "H");
+     *  For example:
      *
-     *  Allows the user to restrict a search with site:xapian.org which
+     *  @code
+     *  qp.add_boolean_prefix("site", "H");
+     *  @endcode
+     *
+     *  This allows the user to restrict a search with site:xapian.org which
      *  will be converted to Hxapian.org combined with any probabilistic
-     *  query with OP_FILTER.
+     *  query with @c Xapian::Query::OP_FILTER.
      *
      *  If multiple boolean filters are specified in a query for the same
-     *  prefix, they will be combined with the OR operator.  Then, if there are
-     *  boolean filters for different prefixes, they will be combined with the
-     *  AND operator.
+     *  prefix, they will be combined with the @c Xapian::Query::OP_OR
+     *  operator.  Then, if there are boolean filters for different prefixes,
+     *  they will be combined with the @c Xapian::Query::OP_AND operator.
      *
-     *  Multiple fields can be mapped to the same prefix (so you can e.g. make
-     *  site: and domain: aliases for each other).  Instances of fields with
-     *  different aliases but the same prefix will still be combined with the
-     *  OR operator.
+     *  Multiple fields can be mapped to the same prefix (so for example
+     *  you can make site: and domain: aliases for each other).  Instances of
+     *  fields with different aliases but the same prefix will still be
+     *  combined with the OR operator.
      *
      *  For example, if "site" and "domain" map to "H", but author maps to "A",
      *  a search for "site:Foo domain:Bar author:Fred" will map to
      *  "(Hfoo OR Hbar) AND Afred".
      *
+     *  As of 1.0.4, you can call this method multiple times with the same
+     *  value of field to allow a single field to be mapped to multiple
+     *  prefixes.  Multiple terms being generated for such a field, and
+     *  combined with @c Xapian::Query::OP_OR.
+     *
+     *  Calling this method with an empty string for @a field will cause
+     *  a @c Xapian::InvalidArgumentError.
+     *
+     *  If you call @c add_prefix() and @c add_boolean_prefix() for the
+     *  same value of @a field, a @c Xapian::InvalidOperationError exception
+     *  will be thrown.
+     *
+     *  In 1.0.3 and earlier, subsequent calls to this method with the same
+     *  value of @a field had no effect.
+     *
      *  @param field   The user visible field name
      *  @param prefix  The term prefix to map this to
      */
