@@ -26,19 +26,19 @@
 #include <xapian/enquire.h>
 
 bool
-EmptySubMatch::prepare_match(bool /*nowait*/) {
+EmptySubMatch::prepare_match(bool /*nowait*/, Stats & /*total_stats*/) {
     return true;
 }
 
 void
-EmptySubMatch::start_match(Xapian::doccount, Xapian::doccount) {
+EmptySubMatch::start_match(Xapian::doccount, Xapian::doccount,
+			   Xapian::doccount, const Stats &)
+{
 }
 
 PostList *
 EmptySubMatch::get_postlist_and_term_info(MultiMatch *,
 	map<string, Xapian::MSet::Internal::TermFreqAndWeight> *)
 {
-    LeafPostList * pl = new EmptyPostList;
-    pl->set_termweight(new Xapian::BoolWeight);
-    return pl;
+    return new EmptyPostList;
 }
