@@ -5,6 +5,7 @@
  * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  * Copyright 2003 Orange PCS Ltd
  * Copyright 2003 Sam Liddicott
+ * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -71,97 +72,75 @@ const Xapian::Enquire::Internal::sort_setting VAL_REL =
 	Xapian::Enquire::Internal::VAL_REL;
 #endif
 
-////////////////////////////////////
-// Initialisation and cleaning up //
-////////////////////////////////////
-MultiMatch::MultiMatch(const Xapian::Database &db_,
-		       const Xapian::Query::Internal * query_,
-		       Xapian::termcount qlen,
-		       const Xapian::RSet & omrset,
-		       Xapian::valueno collapse_key_,
-		       int percent_cutoff_, Xapian::weight weight_cutoff_,
-		       Xapian::Enquire::docid_order order_,
-		       Xapian::valueno sort_key_,
-		       Xapian::Enquire::Internal::sort_setting sort_by_,
-		       bool sort_value_forward_,
-		       Xapian::ErrorHandler * errorhandler_,
-		       StatsGatherer * gatherer_,
-		       const Xapian::Weight * weight_)
-	: gatherer(gatherer_), db(db_), query(query_),
-	  collapse_key(collapse_key_), percent_cutoff(percent_cutoff_),
-	  weight_cutoff(weight_cutoff_), order(order_),
-	  sort_key(sort_key_), sort_by(sort_by_),
-	  sort_value_forward(sort_value_forward_),
-	  errorhandler(errorhandler_), weight(weight_),
-	  is_remote(db.internal.size())
+/** Split an RSet into several sub rsets, one for each database.
+ *
+ *  @param rset The RSet to split.
+ *  @param number_of_subdbs The number of sub databases which exist.
+ *  @param subrsets Vector of RSets which will the sub rsets will be placed in.
+ *                  This should be empty when the function is called.
+ */
+static void
+split_rset_by_db(const Xapian::RSet * rset,
+		 Xapian::doccount number_of_subdbs,
+		 vector<Xapian::RSet> & subrsets)
 {
-    DEBUGCALL(MATCH, void, "MultiMatch", db_ << ", " << query_ << ", " <<
-	      qlen << ", " << omrset << ", " << collapse_key_ << ", " <<
-	      percent_cutoff_ << ", " << weight_cutoff_ << ", " <<
-	      int(order_) << ", " << sort_key_ << ", " <<
-	      int(sort_by_) << ", " << sort_value_forward_ << ", " <<
-	      errorhandler_ << ", [gatherer_], [weight_]");
-    if (!query) return;
+    DEBUGCALL_STATIC(MATCH, void, "split_rset_by_db",
+		     (rset ? *rset : Xapian::RSet()) <<
+		     ", " << number_of_subdbs <<
+		     ", [subrsets(size=" << subrsets.size() << "]");
+    if (rset) {
+	if (number_of_subdbs == 1) {
+	    // The common case of a single database is easy to handle.
+	    subrsets.push_back(*rset);
+	} else {
+	    // Can't just use vector::resize() here, since that creates N
+	    // copies of the same RSet!
+	    subrsets.reserve(number_of_subdbs);
+	    for (size_t i = 0; i < number_of_subdbs; ++i) {
+		subrsets.push_back(Xapian::RSet());
+	    }
 
-    query->validate_query();
-
-    vector<Xapian::RSet> subrsets;
-    if (db.internal.size() == 1) {
-	// Shortcut the common case.
-	subrsets.push_back(omrset);
-    } else {
-	Xapian::doccount number_of_leaves = db.internal.size();
-	// Can't just use resize - that creates N copies of the same RSet!
-	subrsets.reserve(number_of_leaves);
-	for (size_t i = 0; i < number_of_leaves; ++i) {
-	    subrsets.push_back(Xapian::RSet());
-	}
-
-	const set<Xapian::docid> & items = omrset.internal->get_items();
-	set<Xapian::docid>::const_iterator j;
-	for (j = items.begin(); j != items.end(); ++j) {
-	    Xapian::doccount local_docid = (*j - 1) / number_of_leaves + 1;
-	    Xapian::doccount subdatabase = (*j - 1) % number_of_leaves;
-	    subrsets[subdatabase].add_document(local_docid);
-	}
-    }
-
-    Assert(subrsets.size() == db.internal.size());
-    for (size_t i = 0; i != db.internal.size(); ++i) {
-	Xapian::Database::Internal *subdb = db.internal[i].get();
-	Assert(subdb);
-	Xapian::Internal::RefCntPtr<SubMatch> smatch;
-	try {
-	    // There is currently only one special case, for network databases.
-#ifdef XAPIAN_HAS_REMOTE_BACKEND
-	    RemoteDatabase *rem_db = subdb->as_remotedatabase();
-	    if (rem_db) {
-		is_remote[i] = true;
-		rem_db->set_query(query, qlen, collapse_key, order, sort_key,
-				  sort_by, sort_value_forward, percent_cutoff,
-				  weight_cutoff, weight, subrsets[i]);
-		bool decreasing_relevance =
-		    (sort_by == REL || sort_by == REL_VAL);
-		smatch = new RemoteSubMatch(rem_db, gatherer.get(),
-					    decreasing_relevance);
-	    } else {
-#endif /* XAPIAN_HAS_REMOTE_BACKEND */
-		smatch = new LocalSubMatch(subdb, query, qlen, subrsets[i], gatherer.get(), weight);
-#ifdef XAPIAN_HAS_REMOTE_BACKEND
+	    const set<Xapian::docid> & items = rset->internal->get_items();
+	    set<Xapian::docid>::const_iterator j;
+	    for (j = items.begin(); j != items.end(); ++j) {
+		Xapian::doccount local_docid = (*j - 1) / number_of_subdbs + 1;
+		Xapian::doccount subdatabase = (*j - 1) % number_of_subdbs;
+		subrsets[subdatabase].add_document(local_docid);
 	    }
-#endif /* XAPIAN_HAS_REMOTE_BACKEND */
-	} catch (Xapian::Error & e) {
-	    if (!errorhandler) throw;
-	    DEBUGLINE(EXCEPTION, "Calling error handler for creation of a SubMatch from a database and query.");
-	    (*errorhandler)(e);
-	    // Continue match without this sub-postlist.
-	    smatch = new EmptySubMatch;
 	}
-	leaves.push_back(smatch);
+    } else {
+	// NB vector::resize() creates N copies of the same empty RSet.
+	subrsets.resize(number_of_subdbs);
     }
+    Assert(subrsets.size() == number_of_subdbs);
+}
 
-    gatherer->set_global_stats(omrset.size());
-
+/** Prepare some SubMatches.
+ *
+ *  This calls the prepare_match() method on each SubMatch object, causing them
+ *  to lookup various statistics.
+ *
+ *  This method is rather complicated in order to handle remote matches
+ *  efficiently.  Instead of simply calling "prepare_match()" on each submatch
+ *  and waiting for it to return, it first calls "prepare_match(true)" on each
+ *  submatch.  If any of these calls return false, indicating that the required
+ *  information has not yet been received from the server, the method will try
+ *  those which returned false again, passing "false" as a parameter to
+ *  indicate that they should block until the information is ready.
+ *
+ *  This should improve performance in the case of mixed local-and-remote
+ *  searches - the local searchers will all fetch their statistics from disk
+ *  without waiting for the remote searchers, so as soon as the remote searcher
+ *  statistics arrive, we can move on to the next step.
+ */
+static void
+prepare_sub_matches(std::vector<Xapian::Internal::RefCntPtr<SubMatch> > & leaves,
+		    Xapian::ErrorHandler * errorhandler,
+		    Stats & stats)
+{
+    DEBUGCALL_STATIC(MATCH, void, "prepare_sub_matches",
+		     "[leaves(size=" << leaves.size() << ")], " <<
+		     errorhandler << ", " << stats);
     // We use a vector<bool> to track which SubMatches we're already prepared.
     vector<bool> prepared;
     prepared.resize(leaves.size(), false);
@@ -171,7 +150,7 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 	for (size_t leaf = 0; leaf < leaves.size(); ++leaf) {
 	    if (prepared[leaf]) continue;
 	    try {
-		if (leaves[leaf]->prepare_match(nowait)) {
+		if (leaves[leaf]->prepare_match(nowait, stats)) {
 		    prepared[leaf] = true;
 		    --unprepared;
 		}
@@ -192,6 +171,82 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
     }
 }
 
+////////////////////////////////////
+// Initialisation and cleaning up //
+////////////////////////////////////
+MultiMatch::MultiMatch(const Xapian::Database &db_,
+		       const Xapian::Query::Internal * query_,
+		       Xapian::termcount qlen,
+		       const Xapian::RSet * omrset,
+		       Xapian::valueno collapse_key_,
+		       int percent_cutoff_, Xapian::weight weight_cutoff_,
+		       Xapian::Enquire::docid_order order_,
+		       Xapian::valueno sort_key_,
+		       Xapian::Enquire::Internal::sort_setting sort_by_,
+		       bool sort_value_forward_,
+		       const Xapian::Sorter * sorter_,
+		       Xapian::ErrorHandler * errorhandler_,
+		       Stats & stats,
+		       const Xapian::Weight * weight_)
+	: db(db_), query(query_),
+	  collapse_key(collapse_key_), percent_cutoff(percent_cutoff_),
+	  weight_cutoff(weight_cutoff_), order(order_),
+	  sort_key(sort_key_), sort_by(sort_by_),
+	  sort_value_forward(sort_value_forward_), sorter(sorter_),
+	  errorhandler(errorhandler_), weight(weight_),
+	  is_remote(db.internal.size())
+{
+    DEBUGCALL(MATCH, void, "MultiMatch", db_ << ", " << query_ << ", " <<
+	      qlen << ", " << (omrset ? *omrset : Xapian::RSet()) << ", " <<
+	      collapse_key_ << ", " <<
+	      percent_cutoff_ << ", " << weight_cutoff_ << ", " <<
+	      int(order_) << ", " << sort_key_ << ", " <<
+	      int(sort_by_) << ", " << sort_value_forward_ << ", " <<
+	      sorter_ << ", " <<
+	      errorhandler_ << ", " << stats << ", [weight_]");
+
+    if (!query) return;
+    query->validate_query();
+
+    Xapian::doccount number_of_subdbs = db.internal.size();
+    vector<Xapian::RSet> subrsets;
+    split_rset_by_db(omrset, number_of_subdbs, subrsets);
+
+    for (size_t i = 0; i != number_of_subdbs; ++i) {
+	Xapian::Database::Internal *subdb = db.internal[i].get();
+	Assert(subdb);
+	Xapian::Internal::RefCntPtr<SubMatch> smatch;
+	try {
+	    // There is currently only one special case, for network databases.
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
+	    RemoteDatabase *rem_db = subdb->as_remotedatabase();
+	    if (rem_db) {
+		is_remote[i] = true;
+		rem_db->set_query(query, qlen, collapse_key, order, sort_key,
+				  sort_by, sort_value_forward, percent_cutoff,
+				  weight_cutoff, weight, subrsets[i]);
+		bool decreasing_relevance =
+		    (sort_by == REL || sort_by == REL_VAL);
+		smatch = new RemoteSubMatch(rem_db, decreasing_relevance);
+	    } else {
+#endif /* XAPIAN_HAS_REMOTE_BACKEND */
+		smatch = new LocalSubMatch(subdb, query, qlen, subrsets[i], weight);
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
+	    }
+#endif /* XAPIAN_HAS_REMOTE_BACKEND */
+	} catch (Xapian::Error & e) {
+	    if (!errorhandler) throw;
+	    DEBUGLINE(EXCEPTION, "Calling error handler for creation of a SubMatch from a database and query.");
+	    (*errorhandler)(e);
+	    // Continue match without this sub-postlist.
+	    smatch = new EmptySubMatch;
+	}
+	leaves.push_back(smatch);
+    }
+
+    prepare_sub_matches(leaves, errorhandler, stats);
+}
+
 string
 MultiMatch::get_collapse_key(PostList *pl, Xapian::docid did,
 			     Xapian::valueno keyno, Xapian::Internal::RefCntPtr<Xapian::Document::Internal> &doc)
@@ -205,8 +260,7 @@ MultiMatch::get_collapse_key(PostList *pl, Xapian::docid did,
 	Xapian::doccount n = (did - 1) % multiplier; // which actual database
 	Xapian::docid m = (did - 1) / multiplier + 1; // real docid in that database
 
-	Xapian::Internal::RefCntPtr<Xapian::Document::Internal> temp(db.internal[n]->open_document(m, true));
-	doc = temp;
+	doc = db.internal[n]->open_document(m, true);
     }
     RETURN(doc->get_value(keyno));
 }
@@ -231,7 +285,9 @@ MultiMatch::getorrecalc_maxweight(PostList *pl)
 void
 MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		     Xapian::doccount check_at_least,
-		     Xapian::MSet & mset, const Xapian::MatchDecider *mdecider,
+		     Xapian::MSet & mset,
+		     const Stats & stats,
+		     const Xapian::MatchDecider *mdecider,
 		     const Xapian::MatchDecider *matchspy)
 {
     DEBUGCALL(MATCH, void, "MultiMatch::get_mset", first << ", " << maxitems
@@ -245,24 +301,32 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 
     Assert(!leaves.empty());
 
-    // Start matchers
+    // If there's only one database and it's remote, we can just unserialise
+    // its MSet and return that.
+    if (leaves.size() == 1 && is_remote[0]) {
+	RemoteSubMatch * rem_match;
+	rem_match = static_cast<RemoteSubMatch*>(leaves[0].get());
+	rem_match->start_match(first, maxitems, check_at_least, stats);
+	rem_match->get_mset(mset);
+	return;
+    }
+
+    // Start matchers.
     {
 	vector<Xapian::Internal::RefCntPtr<SubMatch> >::iterator leaf;
 	for (leaf = leaves.begin(); leaf != leaves.end(); ++leaf) {
 	    try {
 		// FIXME: look if we can push the "check_at_least" stuff
 		// into the remote match handling too.
-		(*leaf)->start_match(first + maxitems, first + check_at_least);
+		(*leaf)->start_match(0, first + maxitems,
+				     first + check_at_least, stats);
 	    } catch (Xapian::Error & e) {
-		if (errorhandler) {
-		    DEBUGLINE(EXCEPTION, "Calling error handler for "
-			      "start_match() on a SubMatch.");
-		    (*errorhandler)(e);
-		    // Continue match without this sub-match.
-		    *leaf = Xapian::Internal::RefCntPtr<SubMatch>(new EmptySubMatch());
-		} else {
-		    throw;
-		}
+		if (!errorhandler) throw;
+		DEBUGLINE(EXCEPTION, "Calling error handler for "
+			  "start_match() on a SubMatch.");
+		(*errorhandler)(e);
+		// Continue match without this sub-match.
+		*leaf = Xapian::Internal::RefCntPtr<SubMatch>(new EmptySubMatch());
 	    }
 	}
     }
@@ -272,26 +336,41 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     map<string, Xapian::MSet::Internal::TermFreqAndWeight> termfreqandwts;
     map<string, Xapian::MSet::Internal::TermFreqAndWeight> * termfreqandwts_ptr;
     termfreqandwts_ptr = &termfreqandwts;
-    {
-	vector<Xapian::Internal::RefCntPtr<SubMatch> >::iterator leaf;
-	for (leaf = leaves.begin(); leaf != leaves.end(); ++leaf) {
-	    PostList *pl;
-	    try {
-		pl = (*leaf)->get_postlist_and_term_info(this, termfreqandwts_ptr);
-		if (termfreqandwts_ptr && !termfreqandwts.empty())
-		    termfreqandwts_ptr = NULL;
-	    } catch (Xapian::Error & e) {
-		if (!errorhandler) throw;
-		DEBUGLINE(EXCEPTION, "Calling error handler for "
-			  "get_term_info() on a SubMatch.");
-		(*errorhandler)(e);
-		// FIXME: check if *ALL* the remote servers have failed!
-		// Continue match without this sub-match.
-		*leaf = new EmptySubMatch();
-		pl = new EmptyPostList;
+
+    // Keep a count of matches which we know exist, but we won't see.  This
+    // occurs when a submatch is remote, and returns a lower bound on the
+    // number of matching documents which is higher than the number of
+    // documents it returns (because it wasn't asked for more documents).
+    Xapian::doccount definite_matches_not_seen = 0;
+    for (size_t i = 0; i != leaves.size(); ++i) {
+	PostList *pl;
+	try {
+	    pl = leaves[i]->get_postlist_and_term_info(this,
+						       termfreqandwts_ptr);
+	    if (termfreqandwts_ptr && !termfreqandwts.empty())
+		termfreqandwts_ptr = NULL;
+	    if (is_remote[i]) {
+		if (pl->get_termfreq_min() > first + maxitems) {
+		    DEBUGLINE(MATCH, "Found " <<
+			      pl->get_termfreq_min() - (first + maxitems)
+			      << " definite matches in remote "
+			      "submatch which aren't passed to local "
+			      "match");
+		    definite_matches_not_seen += pl->get_termfreq_min();
+		    definite_matches_not_seen -= first + maxitems;
+		}
 	    }
-	    postlists.push_back(pl);
+	} catch (Xapian::Error & e) {
+	    if (!errorhandler) throw;
+	    DEBUGLINE(EXCEPTION, "Calling error handler for "
+		      "get_term_info() on a SubMatch.");
+	    (*errorhandler)(e);
+	    // FIXME: check if *ALL* the remote servers have failed!
+	    // Continue match without this sub-match.
+	    leaves[i] = new EmptySubMatch();
+	    pl = new EmptyPostList;
 	}
+	postlists.push_back(pl);
     }
     Assert(!postlists.empty());
 
@@ -319,8 +398,6 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     Xapian::doccount matches_upper_bound = pl->get_termfreq_max();
     Xapian::doccount matches_lower_bound = pl->get_termfreq_min();
     Xapian::doccount matches_estimated   = pl->get_termfreq_est();
-    Xapian::doccount duplicates_found = 0;
-    Xapian::doccount documents_considered = 0;
 
     // Check if any results have been asked for (might just be wanting
     // maxweight).
@@ -347,6 +424,17 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	return;
     }
 
+    // duplicates_found and documents_considered are used solely to keep track
+    // of how frequently collapses are occurring, so that the matches_estimated
+    // can be adjusted accordingly.
+    Xapian::doccount duplicates_found = 0;
+    Xapian::doccount documents_considered = 0;
+
+    // We keep track of the number of null collapse values because this allows
+    // us to provide a better value for matches_lower_bound if there are null
+    // collapse values.
+    Xapian::doccount null_collapse_values = 0;
+
     // Set max number of results that we want - this is used to decide
     // when to throw away unwanted items.
     Xapian::doccount max_msize = first + maxitems;
@@ -384,28 +472,30 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     bool is_heap = false;
 
     while (true) {
-	if (recalculate_w_max) {
-	    if (min_weight > 0.0 && getorrecalc_maxweight(pl) < min_weight) {
-		DEBUGLINE(MATCH, "*** TERMINATING EARLY (1)");
-		break;
+	if (rare(recalculate_w_max)) {
+	    if (min_weight > 0.0) {
+		if (rare(getorrecalc_maxweight(pl) < min_weight)) {
+		    DEBUGLINE(MATCH, "*** TERMINATING EARLY (1)");
+		    break;
+		}
 	    }
 	}
 
-	if (next_handling_prune(pl, min_weight, this)) {
+	if (rare(next_handling_prune(pl, min_weight, this))) {
 	    DEBUGLINE(MATCH, "*** REPLACING ROOT");
 
 	    if (min_weight > 0.0) {
 		// No need for a full recalc (unless we've got to do one
 		// because of a prune elsewhere) - we're just switching to a
 		// subtree.
-		if (getorrecalc_maxweight(pl) < min_weight) {
+		if (rare(getorrecalc_maxweight(pl) < min_weight)) {
 		    DEBUGLINE(MATCH, "*** TERMINATING EARLY (2)");
 		    break;
 		}
 	    }
 	}
 
-	if (pl->at_end()) {
+	if (rare(pl->at_end())) {
 	    DEBUGLINE(MATCH, "Reached end of potential matches");
 	    break;
 	}
@@ -417,7 +507,10 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	bool calculated_weight = false;
 	if (sort_by != VAL || min_weight > 0.0) {
 	    wt = pl->get_weight();
-	    if (wt < min_weight) continue;
+	    if (wt < min_weight) {
+		DEBUGLINE(MATCH, "Rejecting potential match due to insufficient weight");
+		continue;
+	    }
 	    calculated_weight = true;
 	}
 
@@ -430,13 +523,35 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    Xapian::doccount n = (new_item.did - 1) % multiplier; // which actual database
 	    Xapian::docid m = (new_item.did - 1) / multiplier + 1; // real docid in that database
 	    Xapian::Internal::RefCntPtr<Xapian::Document::Internal> doc(db.internal[n]->open_document(m, true));
-	    new_item.sort_key = doc->get_value(sort_key);
+	    if (sorter) {
+		new_item.sort_key = (*sorter)(Xapian::Document(doc.get()));
+	    } else {
+		new_item.sort_key = doc->get_value(sort_key);
+	    }
 
 	    // We're sorting by value (in part at least), so compare the item
 	    // against the lowest currently in the proto-mset.  If sort_by is
 	    // VAL, then new_item.wt won't yet be set, but that doesn't
 	    // matter since it's not used by the sort function.
-	    if (!mcmp(new_item, min_item)) continue;
+	    if (!mcmp(new_item, min_item)) {
+		if (matchspy == NULL && mdecider == NULL &&
+		    collapse_key == Xapian::BAD_VALUENO) {
+		    // Document was definitely suitable for mset - no more
+		    // processing needed.
+		    DEBUGLINE(MATCH, "Making note of match item which sorts lower than min_item");
+		    ++docs_matched;
+		    continue;
+		}
+		if (docs_matched >= check_at_least) {
+		    // We've seen enough items - we can drop this one.
+		    DEBUGLINE(MATCH, "Dropping candidate which sorts lower than min_item");
+		    continue;
+		}
+		// We can't drop the item, because we need to show it
+		// to the matchspy, test whether the mdecider would
+		// accept it, and/or test whether it would be collapsed.
+		DEBUGLINE(MATCH, "Keeping candidate which sorts lower than min_item for further investigation");
+	    }
 	}
 
 	Xapian::Internal::RefCntPtr<Xapian::Document::Internal> doc;
@@ -476,7 +591,9 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 						     doc);
 
 	    // Don't collapse on null key
-	    if (!new_item.collapse_key.empty()) {
+	    if (new_item.collapse_key.empty()) {
+		++null_collapse_values;
+	    } else {
 		map<string, pair<Xapian::Internal::MSetItem, Xapian::weight> >::iterator oldkey;
 		oldkey = collapse_tab.find(new_item.collapse_key);
 		if (oldkey == collapse_tab.end()) {
@@ -512,7 +629,8 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		    // we've seen so far.  Check if the previous best entry
 		    // with this key might still be in the proto-MSet.  If it
 		    // might be, we need to check through for it.
-		    if (old_item.wt >= min_weight && mcmp(old_item, min_item)) {
+		    Xapian::weight old_wt = old_item.wt;
+		    if (old_wt >= min_weight && mcmp(old_item, min_item)) {
 			// Scan through (unsorted) MSet looking for entry.
 			// FIXME: more efficient way than just scanning?
 			Xapian::docid olddid = old_item.did;
@@ -537,8 +655,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		    }
 
 		    // Keep the old weight as it is now second best so far
-		    oldkey->second = make_pair(new_item,
-					       oldkey->second.first.wt);
+		    oldkey->second = make_pair(new_item, old_wt);
 		}
 	    }
 	}
@@ -550,8 +667,7 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		items.push_back(new_item);
 		if (!is_heap) {
 		    is_heap = true;
-		    make_heap<vector<Xapian::Internal::MSetItem>::iterator,
-			      MSetCmp>(items.begin(), items.end(), mcmp);
+		    make_heap(items.begin(), items.end(), mcmp);
 		} else {
 		    push_heap<vector<Xapian::Internal::MSetItem>::iterator,
 			      MSetCmp>(items.begin(), items.end(), mcmp);
@@ -563,10 +679,27 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		min_item = items.front();
 		if (sort_by == REL || sort_by == REL_VAL) {
 		    if (docs_matched >= check_at_least) {
-			if (min_item.wt > min_weight) min_weight = min_item.wt;
+			if (sort_by == REL) {
+			    // We're done if this is a forward boolean match
+			    // with only one database (bodgetastic, FIXME
+			    // better if we can!)
+			    if (rare(max_weight == 0 && sort_forward)) {
+				// In the multi database case, MergePostList
+				// currently processes each database
+				// sequentially (which actually may well be
+				// more efficient) so the docids in general
+				// won't arrive in order.
+				if (leaves.size() == 1) break;
+			    }
+			}
+			if (min_item.wt > min_weight) {
+			    DEBUGLINE(MATCH, "Setting min_weight to " <<
+				      min_item.wt << " from " << min_weight);
+			    min_weight = min_item.wt;
+			}
 		    }
 		}
-		if (getorrecalc_maxweight(pl) < min_weight) {
+		if (rare(getorrecalc_maxweight(pl) < min_weight)) {
 		    DEBUGLINE(MATCH, "*** TERMINATING EARLY (3)");
 		    break;
 		}
@@ -574,9 +707,19 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 		items.push_back(new_item);
 		is_heap = false;
 		if (sort_by == REL && items.size() == max_msize) {
-		    // We're done if this is a forward boolean match
-		    // (bodgetastic, FIXME better if we can)
-		    if (max_weight == 0 && sort_forward) break;
+		    if (docs_matched >= check_at_least) {
+			// We're done if this is a forward boolean match
+			// with only one database (bodgetastic, FIXME
+			// better if we can!)
+			if (rare(max_weight == 0 && sort_forward)) {
+			    // In the multi database case, MergePostList
+			    // currently processes each database
+			    // sequentially (which actually may well be
+			    // more efficient) so the docids in general
+			    // won't arrive in order.
+			    if (leaves.size() == 1) break;
+			}
+		    }
 		}
 	    }
 	}
@@ -691,12 +834,32 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	percent_scale *= 100.0;
     }
 
+    DEBUGLINE(MATCH,
+	      "docs_matched = " << docs_matched <<
+	      ", definite_matches_not_seen = " << definite_matches_not_seen <<
+	      ", matches_lower_bound = " << matches_lower_bound <<
+	      ", matches_estimated = " << matches_estimated <<
+	      ", matches_upper_bound = " << matches_upper_bound);
+
+    // Adjust docs_matched to take account of documents which matched remotely
+    // but weren't sent across.
+    docs_matched += definite_matches_not_seen;
+
     if (items.size() < max_msize) {
+	// We have fewer items in the mset than we tried to get for it, so we
+	// must have all the matches in it.
 	DEBUGLINE(MATCH, "items.size() = " << items.size() <<
 		  ", max_msize = " << max_msize << ", setting bounds equal");
+	Assert(definite_matches_not_seen == 0);
 	Assert(percent_cutoff || docs_matched == items.size());
 	matches_lower_bound = matches_upper_bound = matches_estimated
 	    = items.size();
+    } else if (docs_matched < check_at_least) {
+	// We have seen fewer matches than we checked for, so we must have seen
+	// all the matches.
+	DEBUGLINE(MATCH, "Setting bounds equal");
+	matches_lower_bound = matches_upper_bound = matches_estimated
+	    = docs_matched;
     } else {
 	if (percent_cutoff) {
 	    // another approach: Xapian::doccount new_est = items.size() * (1 - percent_factor) / (1 - min_weight / greatest_wt);
@@ -705,30 +868,25 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    matches_estimated = max(size_t(new_est), items.size());
 	    // and another: items.size() + (1 - greatest_wt * percent_factor / min_weight) * (matches_estimated - items.size());
 
-	    // Very likely an underestimate, but we can't really do better without
-	    // checking further matches...  Only possibility would be to track how
-	    // many docs made the min weight test but didn't make the candidate set
-	    // since the last greatest_wt change, which we could use if the top
-	    // documents matched all the prob terms.
+	    // Very likely an underestimate, but we can't really do better
+	    // without checking further matches...  Only possibility would be
+	    // to track how many docs made the min weight test but didn't make
+	    // the candidate set since the last greatest_wt change, which we
+	    // could use if the top documents matched all the prob terms.
 	    matches_lower_bound = items.size();
 	    // matches_upper_bound could be reduced by the number of documents
 	    // which fail the min weight test
 	}
 
-	DEBUGLINE(MATCH,
-		"docs_matched = " << docs_matched << ", " <<
-		"matches_lower_bound = " << matches_lower_bound << ", " <<
-		"matches_estimated = " << matches_estimated << ", " <<
-		"matches_upper_bound = " << matches_upper_bound);
-
 	Assert(matches_estimated >= matches_lower_bound);
 	Assert(matches_estimated <= matches_upper_bound);
 
 	if (collapse_key != Xapian::BAD_VALUENO) {
-	    // Lower bound must be set to no more than the number of collapse
-	    // values we've got, since it's possible that all further hits
-	    // will be collapsed to a single value.
-	    matches_lower_bound = collapse_tab.size();
+	    // Lower bound must be set to no more than the number of null
+	    // collapse values seen plus the number of unique non-null collapse
+	    // values seen, since it's possible that all further hits will be
+	    // collapsed to values we've already seen.
+	    matches_lower_bound = null_collapse_values + collapse_tab.size();
 
 	    // The estimate for the number of hits can be modified by
 	    // multiplying it by the rate at which we've been finding
@@ -764,23 +922,26 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	    items.clear();
 	} else {
 	    DEBUGLINE(MATCH, "finding " << first << "th");
-	    nth_element(items.begin(), items.begin() + first, items.end(),
-			mcmp);
-	    // Erase the leading ``first'' elements
-	    items.erase(items.begin(), items.begin() + first);
+	    // We perform nth_element() on reverse iterators so that the
+	    // unwanted elements end up at the end of items, which means
+	    // that the call to erase() to remove them doesn't have to copy
+	    // any elements.
+	    vector<Xapian::Internal::MSetItem>::reverse_iterator nth;
+	    nth = items.rbegin() + first;
+	    nth_element(items.rbegin(), nth, items.rend(), mcmp);
+	    // Erase the trailing ``first'' elements
+	    items.erase(items.begin() + items.size() - first, items.end());
 	}
     }
 
-    DEBUGLINE(MATCH, "msize = " << items.size());
+    DEBUGLINE(MATCH, "sorting " << items.size() << " entries");
+
+    // Need a stable sort, but this is provided by comparison operator
+    sort(items.begin(), items.end(), mcmp);
 
     if (!items.empty()) {
-	DEBUGLINE(MATCH, "sorting");
-
-	// Need a stable sort, but this is provided by comparison operator
-	sort(items.begin(), items.end(), mcmp);
-
-	DEBUGLINE(MATCH, "max weight in mset = " << items[0].wt <<
-		  ", min weight in mset = " << items.back().wt);
+	DEBUGLINE(MATCH, "min weight in mset = " << items.back().wt);
+	DEBUGLINE(MATCH, "max weight in mset = " << items[0].wt);
     }
 
     Assert(matches_estimated >= matches_lower_bound);
@@ -791,8 +952,8 @@ MultiMatch::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
     // We WILL need to restore collapse_count to the mset by taking from
     // collapse_tab; this is what comes of copying around whole objects
     // instead of taking references, we find it hard to update collapse_count
-    // of an item that has already been pushed-back as we don't know where it is
-    // any more.  If we keep or find references we won't need to mess with
+    // of an item that has already been pushed-back as we don't know where it
+    // is any more.  If we keep or find references we won't need to mess with
     // is_heap so much maybe?
     if (collapse_key != Xapian::BAD_VALUENO && /*percent_cutoff &&*/ !items.empty() &&
 	!collapse_tab.empty()) {
