@@ -22,12 +22,13 @@
 
 #include <config.h>
 
+#include <xapian/error.h>
+
 #include "safeerrno.h"
 #include "safefcntl.h"
 
 #include "remoteserver.h"
 #include "tcpserver.h"
-#include "stats.h"
 #include "utils.h"
 
 #ifdef __WIN32__
@@ -44,9 +45,11 @@
 # include <sys/wait.h>
 #endif
 
-#include <sys/types.h>
 #include <iostream>
 
+#include <string.h>
+#include <sys/types.h>
+
 using namespace std;
 
 // Handle older systems.
