@@ -28,6 +28,8 @@
 #include "queryparser_internal.h"
 #include "vectortermlist.h"
 
+#include <string.h>
+
 using namespace Xapian;
 
 using namespace std;
@@ -126,15 +128,19 @@ QueryParser::parse_query(const string &query_string, unsigned flags,
 void
 QueryParser::add_prefix(const string &field, const string &prefix)
 {
-    internal->prefixes.insert(
-	make_pair(field, PrefixInfo(PrefixInfo::FREE_TEXT, prefix)));
+    Assert(internal.get());
+    internal->add_prefix(field, prefix, false);
 }
 
 void
 QueryParser::add_boolean_prefix(const string &field, const string &prefix)
 {
-    internal->prefixes.insert(
-	make_pair(field, PrefixInfo(PrefixInfo::BOOL_FILTER, prefix)));
+    Assert(internal.get());
+    // Don't allow the empty prefix to be set as boolean as it doesn't
+    // really make sense.
+    if (field.empty())
+	throw Xapian::UnimplementedError("Can't set the empty prefix to be a boolean filter");
+    internal->add_prefix(field, prefix, true);
 }
 
 TermIterator
