@@ -5,6 +5,7 @@
  * Copyright 2002 Ananova Ltd
  * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
  * Copyright 2006 Richard Boulton
+ * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +24,9 @@
  */
 
 #include <config.h>
+
+#include "api_wrdb.h"
+
 #include <algorithm>
 #include <map>
 #include <string>
@@ -44,9 +48,8 @@ using namespace std;
 
 // test that indexing a term more than once at the same position increases
 // the wdf
-static bool test_adddoc1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(adddoc1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1, doc2, doc3;
 
@@ -90,9 +93,8 @@ static bool test_adddoc1()
 }
 
 // test that removing a posting and removing a term works
-static bool test_adddoc2()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(adddoc2, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -266,9 +268,8 @@ static bool test_adddoc2()
 
 // test that adding lots of documents works, and doesn't leak memory
 // REGRESSION FIXED:2003-09-07
-static bool test_adddoc3()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(adddoc3, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     for (Xapian::doccount i = 0; i < 2100; ++i) {
 	Xapian::Document doc;
@@ -282,10 +283,218 @@ static bool test_adddoc3()
     return true;
 }
 
+// We want to test that a termlist starting with a 48 character long term works
+// OK since this value currently requires special handling in flint for
+// historical reasons!)  Also test all other term lengths while we're at it.
+DEFINE_TESTCASE(adddoc4, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
+
+    for (Xapian::doccount i = 1; i <= 240; ++i) {
+	Xapian::Document doc;
+	string term(i, 'X');
+	doc.add_term(term);
+	db.add_document(doc);
+    }
+    db.add_document(Xapian::Document());
+    db.flush();
+
+    for (Xapian::doccount i = 1; i <= 240; ++i) {
+	Xapian::Document doc = db.get_document(i);
+	Xapian::TermIterator t = doc.termlist_begin();
+	TEST(t != doc.termlist_end());
+	TEST_EQUAL((*t).size(), i);
+	++t;
+	TEST(t == doc.termlist_end());
+    }
+
+    // And test a document with no terms.
+    Xapian::Document doc = db.get_document(241);
+    TEST(doc.termlist_begin() == doc.termlist_end());
+
+    return true;
+}
+
+// Test adding a document, and checking that it got added correctly.
+// This testcase used to be adddoc2 in quartztest.
+DEFINE_TESTCASE(adddoc5, writable) {
+    // Inmemory doesn't support get_writable_database_as_database().
+    SKIP_TEST_FOR_BACKEND("inmemory");
+
+    Xapian::docid did;
+    Xapian::Document document_in;
+    document_in.set_data("Foobar rising");
+    document_in.add_value(7, "Value7");
+    document_in.add_value(13, "Value13");
+    document_in.add_posting("foobar", 1);
+    document_in.add_posting("rising", 2);
+    document_in.add_posting("foobar", 3);
+
+    Xapian::Document document_in2;
+    document_in2.set_data("Foobar falling");
+    document_in2.add_posting("foobar", 1);
+    document_in2.add_posting("falling", 2);
+    {
+	Xapian::WritableDatabase database(get_writable_database());
+
+	TEST_EQUAL(database.get_doccount(), 0);
+	TEST_EQUAL(database.get_avlength(), 0);
+
+	did = database.add_document(document_in);
+	TEST_EQUAL(database.get_doccount(), 1);
+	TEST_EQUAL(database.get_avlength(), 3);
+
+	TEST_EQUAL(database.get_termfreq("foobar"), 1);
+	TEST_EQUAL(database.get_collection_freq("foobar"), 2);
+	TEST_EQUAL(database.get_termfreq("rising"), 1);
+	TEST_EQUAL(database.get_collection_freq("rising"), 1);
+	TEST_EQUAL(database.get_termfreq("falling"), 0);
+	TEST_EQUAL(database.get_collection_freq("falling"), 0);
+
+	Xapian::docid did2 = database.add_document(document_in2);
+	TEST_EQUAL(database.get_doccount(), 2);
+	TEST_NOT_EQUAL(did, did2);
+	TEST_EQUAL(database.get_avlength(), 5.0/2.0);
+
+	TEST_EQUAL(database.get_termfreq("foobar"), 2);
+	TEST_EQUAL(database.get_collection_freq("foobar"), 3);
+	TEST_EQUAL(database.get_termfreq("rising"), 1);
+	TEST_EQUAL(database.get_collection_freq("rising"), 1);
+	TEST_EQUAL(database.get_termfreq("falling"), 1);
+	TEST_EQUAL(database.get_collection_freq("falling"), 1);
+
+	database.delete_document(did);
+	TEST_EQUAL(database.get_doccount(), 1);
+	TEST_EQUAL(database.get_avlength(), 2);
+
+	TEST_EQUAL(database.get_termfreq("foobar"), 1);
+	TEST_EQUAL(database.get_collection_freq("foobar"), 1);
+	TEST_EQUAL(database.get_termfreq("rising"), 0);
+	TEST_EQUAL(database.get_collection_freq("rising"), 0);
+	TEST_EQUAL(database.get_termfreq("falling"), 1);
+	TEST_EQUAL(database.get_collection_freq("falling"), 1);
+
+	did = database.add_document(document_in);
+	TEST_EQUAL(database.get_doccount(), 2);
+	TEST_EQUAL(database.get_avlength(), 5.0/2.0);
+
+	TEST_EQUAL(database.get_termfreq("foobar"), 2);
+	TEST_EQUAL(database.get_collection_freq("foobar"), 3);
+	TEST_EQUAL(database.get_termfreq("rising"), 1);
+	TEST_EQUAL(database.get_collection_freq("rising"), 1);
+	TEST_EQUAL(database.get_termfreq("falling"), 1);
+	TEST_EQUAL(database.get_collection_freq("falling"), 1);
+    }
+
+    {
+	Xapian::Database database(get_writable_database_as_database());
+	Xapian::Document document_out = database.get_document(did);
+
+	TEST_EQUAL(document_in.get_data(), document_out.get_data());
+
+	{
+	    Xapian::ValueIterator i(document_in.values_begin());
+	    Xapian::ValueIterator j(document_out.values_begin());
+	    for (; i != document_in.values_end(); i++, j++) {
+		TEST_NOT_EQUAL(j, document_out.values_end());
+		TEST_EQUAL(*i, *j);
+		TEST_EQUAL(i.get_valueno(), j.get_valueno());
+	    }
+	    TEST_EQUAL(j, document_out.values_end());
+	}
+
+	{
+	    // Regression test for bug fixed in 1.0.5 - values_begin() didn't
+	    // ensure that values had been read.  However, values_end() did
+	    // (and so did values_count()) so this wasn't generally an issue
+	    // but it shouldn't happen anyway.
+	    Xapian::Document doc_tmp = database.get_document(did);
+	    Xapian::ValueIterator i = document_in.values_begin();
+	    Xapian::ValueIterator j = doc_tmp.values_begin();
+	    TEST_EQUAL(*i, *j);
+	}
+
+	{
+	    Xapian::TermIterator i(document_in.termlist_begin());
+	    Xapian::TermIterator j(document_out.termlist_begin());
+	    for (; i != document_in.termlist_end(); i++, j++) {
+		TEST_NOT_EQUAL(j, document_out.termlist_end());
+		TEST_EQUAL(*i, *j);
+		TEST_EQUAL(i.get_wdf(), j.get_wdf());
+		TEST_EXCEPTION(Xapian::InvalidOperationError,
+			       (void)i.get_termfreq());
+		TEST_NOT_EQUAL(0, j.get_termfreq());
+		if (*i == "foobar") {
+		    // termfreq of foobar is 2
+		    TEST_EQUAL(2, j.get_termfreq());
+		} else {
+		    // termfreq of rising is 1
+		    TEST_EQUAL(*i, "rising");
+		    TEST_EQUAL(1, j.get_termfreq());
+		}
+		Xapian::PositionIterator k(i.positionlist_begin());
+		Xapian::PositionIterator l(j.positionlist_begin());
+		for (; k != i.positionlist_end(); k++, l++) {
+		    TEST_NOT_EQUAL(l, j.positionlist_end());
+		    TEST_EQUAL(*k, *l);
+		}
+		TEST_EQUAL(l, j.positionlist_end());
+	    }
+	    TEST_EQUAL(j, document_out.termlist_end());
+	}
+    }
+
+    return true;
+}
+
+// Test adding a document, and checking that it got added correctly.
+// This testcase used to be adddoc3 in quartztest.
+DEFINE_TESTCASE(adddoc6, writable) {
+    // Inmemory doesn't support get_writable_database_again().
+    SKIP_TEST_FOR_BACKEND("inmemory");
+
+    Xapian::docid did;
+    Xapian::Document document_in;
+    document_in.set_data("Foobar rising");
+    document_in.add_value(7, "Value7");
+    document_in.add_value(13, "Value13");
+    document_in.add_posting("foo", 1);
+    document_in.add_posting("bar", 2);
+
+    {
+	Xapian::WritableDatabase database(get_writable_database());
+
+	did = database.add_document(document_in);
+	TEST_EQUAL(did, 1);
+	TEST_EQUAL(database.get_doccount(), 1);
+	TEST_EQUAL(database.get_avlength(), 2);
+    }
+
+    {
+	Xapian::WritableDatabase database(get_writable_database_again());
+
+	document_in.remove_term("foo");
+	document_in.add_posting("baz", 1);
+
+	database.replace_document(1, document_in);
+
+	database.delete_document(1);
+
+	TEST_EQUAL(database.get_doccount(), 0);
+	TEST_EQUAL(database.get_avlength(), 0);
+	TEST_EQUAL(database.get_termfreq("foo"), 0);
+	TEST_EQUAL(database.get_collection_freq("foo"), 0);
+	TEST_EQUAL(database.get_termfreq("bar"), 0);
+	TEST_EQUAL(database.get_collection_freq("bar"), 0);
+	TEST_EQUAL(database.get_termfreq("baz"), 0);
+	TEST_EQUAL(database.get_collection_freq("baz"), 0);
+    }
+
+    return true;
+}
+
 // tests that database destructors flush if it isn't done explicitly
-static bool test_implicitendsession1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(implicitendsession1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc;
 
@@ -299,9 +508,8 @@ static bool test_implicitendsession1()
 }
 
 // tests that assignment of Xapian::Database and Xapian::WritableDatabase work as expected
-static bool test_databaseassign1()
-{
-    Xapian::WritableDatabase wdb = get_writable_database("");
+DEFINE_TESTCASE(databaseassign1, writable) {
+    Xapian::WritableDatabase wdb = get_writable_database();
     Xapian::Database db = get_database("");
     Xapian::Database actually_wdb = wdb;
     Xapian::WritableDatabase w1(wdb);
@@ -316,9 +524,8 @@ static bool test_databaseassign1()
 }
 
 // tests that deletion and updating of documents works as expected
-static bool test_deldoc1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -364,9 +571,8 @@ static bool test_deldoc1()
 }
 
 // tests that deletion and updating of documents works as expected
-static bool test_deldoc2()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc2, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -441,9 +647,8 @@ static bool test_deldoc2()
 }
 
 // another test of deletion of documents, a cut-down version of deldoc2
-static bool test_deldoc3()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc3, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -490,9 +695,8 @@ static bool test_deldoc3()
 }
 
 // tests that deletion and updating of (lots of) documents works as expected
-static bool test_deldoc4()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc4, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -576,9 +780,8 @@ static bool test_deldoc4()
 }
 
 // Test deleting a document which was added in the same batch.
-static bool test_deldoc5()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc5, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -622,9 +825,8 @@ static bool test_deldoc5()
 }
 
 // Regression test for bug in quartz and flint, fixed in 1.0.2.
-static bool test_deldoc6()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(deldoc6, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -653,9 +855,8 @@ static bool test_deldoc6()
     return true;
 }
 
-static bool test_replacedoc1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(replacedoc1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -700,9 +901,8 @@ static bool test_replacedoc1()
 
 // Test of new feature: WritableDatabase::replace_document accepts a docid
 // which doesn't yet exist as of Xapian 0.8.2.
-static bool test_replacedoc2()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(replacedoc2, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -759,9 +959,8 @@ static bool test_replacedoc2()
 }
 
 // Test replacing a document which was added in the same batch.
-static bool test_replacedoc3()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(replacedoc3, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -826,9 +1025,8 @@ static bool test_replacedoc3()
 }
 
 // Test replacing a document which was deleted in the same batch.
-static bool test_replacedoc4()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(replacedoc4, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
 
@@ -889,9 +1087,8 @@ static bool test_replacedoc4()
 // Test replacing a document with itself without modifying postings.
 // Regression test for bug in 0.9.9 and earlier - there flint and quartz
 // lose all positional information for the document when you do this.
-static bool test_replacedoc5()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(replacedoc5, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc1;
     doc1.add_posting("hello", 1);
@@ -917,9 +1114,8 @@ static bool test_replacedoc5()
 
 // Test of new feature: WritableDatabase::replace_document and delete_document
 // can take a unique termname instead of a document id as of Xapian 0.8.2.
-static bool test_uniqueterm1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(uniqueterm1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     for (int n = 1; n <= 20; ++n) {
 	Xapian::Document doc;
@@ -973,8 +1169,7 @@ static bool test_uniqueterm1()
 }
 
 // tests all document postlists
-static bool test_allpostlist2()
-{
+DEFINE_TESTCASE(allpostlist2, writable) {
     Xapian::WritableDatabase db(get_writable_database("apitest_manydocs"));
     Xapian::PostingIterator i = db.postlist_begin("");
     unsigned int j = 1;
@@ -1027,8 +1222,7 @@ static void test_emptyterm2_helper(Xapian::WritableDatabase & db)
 
 // tests results of passing an empty term to various methods
 // equivalent of emptyterm1 for a writable database
-static bool test_emptyterm2()
-{
+DEFINE_TESTCASE(emptyterm2, writable) {
     {
 	Xapian::WritableDatabase db(get_writable_database("apitest_manydocs"));
 	TEST_EQUAL(db.get_doccount(), 512);
@@ -1054,7 +1248,7 @@ static bool test_emptyterm2()
     }
 
     {
-	Xapian::WritableDatabase db(get_writable_database(""));
+	Xapian::WritableDatabase db(get_writable_database());
 	TEST_EQUAL(db.get_doccount(), 0);
 	test_emptyterm2_helper(db);
     }
@@ -1064,9 +1258,8 @@ static bool test_emptyterm2()
 
 // Check that PHRASE/NEAR becomes AND if there's no positional info in the
 // database.
-static bool test_phraseorneartoand1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(phraseorneartoand1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     for (int n = 1; n <= 20; ++n) {
 	Xapian::Document doc;
@@ -1104,9 +1297,8 @@ static bool test_phraseorneartoand1()
 
 // Check that a large number of position list entries for a particular term
 // works - regression test for flint.
-static bool test_longpositionlist1()
-{
-    Xapian::WritableDatabase db = get_writable_database("");
+DEFINE_TESTCASE(longpositionlist1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
 
     Xapian::Document doc;
     Xapian::termpos n;
@@ -1172,7 +1364,7 @@ static bool test_longpositionlist1()
 // set_sort_by_value_then_relevance.
 bool test_consistency2()
 {
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
     char buf[2] = "X";
     int i = 0;
 
@@ -1222,8 +1414,7 @@ bool test_consistency2()
     return true;
 }
 
-static bool test_crashrecovery1()
-{
+DEFINE_TESTCASE(crashrecovery1, writable) {
     const string & dbtype = get_dbtype();
     string path, base_ext;
     if (dbtype == "flint") {
@@ -1238,8 +1429,8 @@ static bool test_crashrecovery1()
 
     Xapian::Document doc;
     {
-	Xapian::WritableDatabase db = get_writable_database("");
-	Xapian::Database dbr(path);
+	Xapian::WritableDatabase db = get_writable_database();
+	Xapian::Database dbr(get_writable_database_as_database());
 	TEST_EQUAL(dbr.get_doccount(), 0);
 
 	// Xapian::Database has full set of baseA, no baseB
@@ -1288,16 +1479,12 @@ static bool test_crashrecovery1()
 
 // Check that DatabaseError is thrown if the docid counter would wrap.
 // Regression test for bug#152.
-static bool test_nomoredocids1()
-{
-    if (get_dbtype() == "inmemory") {
-	// The InMemory backend uses a vector for the documents, so trying
-	// to add document "-1" will fail because we can't allocate enough
-	// memory!
-	SKIP_TEST("Test not supported on inmemory backend");
-    }
+DEFINE_TESTCASE(nomoredocids1, writable) {
+    // The InMemory backend uses a vector for the documents, so trying to add
+    // document "-1" will fail because we can't allocate enough memory!
+    SKIP_TEST_FOR_BACKEND("inmemory");
 
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
     Xapian::Document doc;
     doc.set_data("prose");
     doc.add_term("word");
@@ -1310,23 +1497,17 @@ static bool test_nomoredocids1()
 }
 
 // Test basic spelling correction features.
-static bool test_spell1()
-{
-    string dbpath;
-    if (get_dbtype() == "flint") {
-	dbpath = ".flint/dbw";
-    } else {
-	SKIP_TEST("Test only supported for flint backend");
-    }
+DEFINE_TESTCASE(spell1, spelling) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
 
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
 
     // Check that the more frequent term is chosen.
     db.add_spelling("hello");
     db.add_spelling("cell", 2);
     TEST_EQUAL(db.get_spelling_suggestion("hell"), "cell");
     db.flush();
-    Xapian::Database dbr(dbpath);
+    Xapian::Database dbr(get_writable_database_as_database());
     TEST_EQUAL(db.get_spelling_suggestion("hell"), "cell");
     TEST_EQUAL(dbr.get_spelling_suggestion("hell"), "cell");
 
@@ -1390,16 +1571,10 @@ static bool test_spell1()
 }
 
 // Test spelling correction for Unicode.
-static bool test_spell2()
-{
-    string dbpath;
-    if (get_dbtype() == "flint") {
-	dbpath = ".flint/dbw";
-    } else {
-	SKIP_TEST("Test only supported for flint backend");
-    }
+DEFINE_TESTCASE(spell2, spelling) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
 
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
 
     // Check that a UTF-8 sequence counts as a single character.
     db.add_spelling("h\xc3\xb6hle");
@@ -1411,7 +1586,7 @@ static bool test_spell2()
     TEST_EQUAL(db.get_spelling_suggestion("as\xc3\xb6\xc3\xb7i"), "ascii");
     TEST_EQUAL(db.get_spelling_suggestion("asc\xc3\xb6i\xc3\xb7i"), "ascii");
     db.flush();
-    Xapian::Database dbr(dbpath);
+    Xapian::Database dbr(get_writable_database_as_database());
     TEST_EQUAL(dbr.get_spelling_suggestion("hohle", 1), "h\xc3\xb6hle");
     TEST_EQUAL(dbr.get_spelling_suggestion("hhle", 1), "h\xc3\xb6hle");
     TEST_EQUAL(dbr.get_spelling_suggestion("\xf0\xa8\xa8\x8f\xc3\xb6le", 2), "h\xc3\xb6hle");
@@ -1423,26 +1598,14 @@ static bool test_spell2()
 }
 
 // Test spelling correction with multi databases
-static bool test_spell3()
-{
-    if (get_dbtype() != "flint") {
-	SKIP_TEST("Test only supported for flint backend");
-    }
+DEFINE_TESTCASE(spell3, spelling) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
 
-#ifdef __WIN32__
-    SKIP_TEST("Test not supported on windows");
-#endif
-    // FIXME: the following two lines create two writable databases, but at the
-    // same path.  The first database is deleted, but kept open, before the
-    // second is opened.  This isn't really supported behaviour (in Olly's
-    // words: "it'll end in tears if it tries to flush"), and doesn't work on
-    // windows (hence the SKIP_TEST for __WIN32__ above) so we need to fix
-    // this.  This probably involves making the backendmanager support a "name"
-    // parameter for get_writable_database() which specifies the path to create
-    // the database at.  This would also make it easier to get a readonly
-    // handle on a previously created writable database.
-    Xapian::WritableDatabase db1 = get_writable_database("");
-    Xapian::WritableDatabase db2 = get_writable_database("");
+    Xapian::WritableDatabase db1 = get_writable_database();
+    // We can't just call get_writable_database() since it would delete db1
+    // which doesn't work at all under __WIN32__ and will go wrong elsewhere if
+    // changes to db1 are committed.
+    Xapian::WritableDatabase db2 = get_named_writable_database("spell3", "");
 
     db1.add_spelling("hello");
     db1.add_spelling("cell", 2);
@@ -1494,13 +1657,10 @@ static bool test_spell3()
 }
 
 // Regression test - check that appending works correctly.
-static bool test_spell4()
-{
-    if (get_dbtype() != "flint") {
-	SKIP_TEST("Test only supported for flint backend");
-    }
+DEFINE_TESTCASE(spell4, spelling) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
 
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
 
     db.add_spelling("check");
     db.add_spelling("pecks", 2);
@@ -1513,14 +1673,25 @@ static bool test_spell4()
     return true;
 }
 
-// Test synonym iterators.
-static bool test_synonymitor1()
-{
-    if (get_dbtype() != "flint") {
-	SKIP_TEST("Test only supported for flint backend");
-    }
+// Regression test - used to segfault with some input values.
+DEFINE_TESTCASE(spell5, spelling) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
 
-    Xapian::WritableDatabase db = get_writable_database("");
+    Xapian::WritableDatabase db = get_writable_database();
+
+    db.add_spelling("一些");
+    db.flush();
+
+    TEST_EQUAL(db.get_spelling_suggestion("不", strlen("不")), "一些");
+
+    return true;
+}
+
+// Test synonym iterators.
+DEFINE_TESTCASE(synonymitor1, writable) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
+
+    Xapian::WritableDatabase db = get_writable_database();
 
     // Test iterators for terms which aren't there.
     TEST(db.synonyms_begin("abc") == db.synonyms_end("abc"));
@@ -1632,217 +1803,154 @@ static bool test_synonymitor1()
     return true;
 }
 
-static bool test_matchspy1()
-{
-    if (get_dbtype() == "remotetcp" || get_dbtype() == "remoteprog") {
-	SKIP_TEST("Test not supported for remote backend");
+// Test basic metadata access methods.
+DEFINE_TESTCASE(metadata1, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
+
+    TEST_EQUAL(db.get_metadata("foo"), "");
+    try {
+	db.set_metadata("foo", "bar");
+    } catch (Xapian::UnimplementedError &) {
+	SKIP_TEST("Metadata not supported by this backend");
     }
+    TEST_EQUAL(db.get_metadata("foo"), "bar");
+    db.set_metadata("foo", "baz");
+    TEST_EQUAL(db.get_doccount(), 0);
+    TEST_EQUAL(db.get_metadata("foo"), "baz");
+    db.set_metadata("foo", "");
+    TEST_EQUAL(db.get_metadata("foo"), "");
 
-    Xapian::WritableDatabase db = get_writable_database("");
-    for (int c = 1; c <= 25; ++c) {
-	Xapian::Document doc;
-	doc.set_data("Document " + om_tostring(c));
-	int factors = 0;
-	for (int factor = 1; factor <= c; ++factor) {
-	    doc.add_term("all");
-	    if (c % factor == 0) {
-		doc.add_term("XFACT" + om_tostring(factor));
-		++factors;
-	    }
-	}
-
-	// Number of factors.
-	doc.add_value(0, om_tostring(factors));
-	// Units digits.
-	doc.add_value(1, om_tostring(c % 10));
-	// Constant.
-	doc.add_value(2, "fish");
-	// Number of digits.
-	doc.add_value(3, om_tostring(om_tostring(c).size()));
-
-	db.add_document(doc);
-    }
-
-    Xapian::MatchSpy spy;
-
-    spy.add_category(0);
-    spy.add_category(1);
-    spy.add_category(3);
-
-    Xapian::Enquire enq(db);
-
-    enq.set_query(Xapian::Query("all"));
-
-    Xapian::MSet mset = enq.get_mset(0, 10, 0, NULL, NULL, &spy);
-
-    TEST_EQUAL(spy.get_total(), 25);
-
-    static const char * results[] = {
-	"|1:1|2:9|3:3|4:7|5:1|6:3|8:1|",
-	"|0:2|1:3|2:3|3:3|4:3|5:3|6:2|7:2|8:2|9:2|",
-	"|",
-	"|1:9|2:16|",
-	"|",
-	NULL
-    };
-    for (Xapian::valueno v = 0; results[v]; ++v) {
-	const map<string, size_t> & cat = spy.get_categories(v);
-	string str("|");
-	map<string, size_t>::const_iterator i;
-	for (i = cat.begin(); i != cat.end(); ++i) {
-	    str += i->first;
-	    str += ':';
-	    str += om_tostring(i->second);
-	    str += '|';
-	}
-	tout << "value " << v << endl;
-	TEST_STRINGS_EQUAL(str, results[v]);
-    }
-
-    {
-	// Test scoring evenness returns scores with the natural ordering.
-	double score0 = spy.score_categorisation(0);
-	tout << "score0 = " << score0 << endl;
-	double score1 = spy.score_categorisation(1);
-	tout << "score1 = " << score1 << endl;
-	double score3 = spy.score_categorisation(3);
-	tout << "score3 = " << score3 << endl;
-	// 1 is obviously best, and 0 obviously worst.
-	TEST(score1 < score3);
-	TEST(score3 < score0);
-    }
-
-    {
-	// Test scoring evenness and about 7 categories returns scores with the
-	// natural ordering.
-	double score0 = spy.score_categorisation(0, 7);
-	tout << "score0 = " << score0 << endl;
-	double score1 = spy.score_categorisation(1, 7);
-	tout << "score1 = " << score1 << endl;
-	double score3 = spy.score_categorisation(3, 7);
-	tout << "score3 = " << score3 << endl;
-	// 3 is clearly worst - 0 is arguably a little better than 1 (0 is the
-	// requested size, but 1 has a much more even split).
-	TEST(score0 < score1);
-	TEST(score1 < score3);
-    }
+    TEST_EQUAL(db.get_doccount(), 0);
 
     return true;
 }
 
-static bool test_matchspy2()
-{
-    if (get_dbtype() == "remotetcp" || get_dbtype() == "remoteprog") {
-	SKIP_TEST("Test not supported for remote backend");
-    }
+// Test that metadata gets applied at same time as other changes.
+DEFINE_TESTCASE(metadata2, metadata) {
+    SKIP_TEST_UNLESS_BACKEND("flint");
+    Xapian::WritableDatabase db = get_writable_database();
+    Xapian::Database dbr = get_writable_database_as_database();
 
-    Xapian::WritableDatabase db = get_writable_database("");
-    for (int c = 1; c <= 25; ++c) {
-	Xapian::Document doc;
-	doc.set_data("Document " + om_tostring(c));
-	int factors = 0;
-	for (int factor = 1; factor <= c; ++factor) {
-	    doc.add_term("all");
-	    if (c % factor == 0) {
-		doc.add_term("XFACT" + om_tostring(factor));
-		++factors;
-	    }
-	}
+    TEST_EQUAL(db.get_metadata("foo"), "");
+    db.set_metadata("foo", "bar");
+    TEST_EQUAL(db.get_metadata("foo"), "bar");
+    TEST_EQUAL(dbr.get_metadata("foo"), "");
+    db.flush();
+    TEST_EQUAL(dbr.get_metadata("foo"), "");
+    dbr.reopen();
+    TEST_EQUAL(db.get_metadata("foo"), "bar");
+    TEST_EQUAL(dbr.get_metadata("foo"), "bar");
+    TEST_EQUAL(dbr.get_doccount(), 0);
 
-	// Number of factors.
-	doc.add_value(0, Xapian::sortable_serialise(factors));
-	// Units digits.
-	doc.add_value(1, Xapian::sortable_serialise(c % 10));
-	// (x + 1/3)*(x + 1/3).
-	doc.add_value(2, Xapian::sortable_serialise((c + 1.0/3.0) * (c + 1.0/3.0)));
-	// Reciprocal.
-	doc.add_value(3, Xapian::sortable_serialise(1.0 / c));
+    db.add_document(Xapian::Document());
+    db.set_metadata("foo", "baz");
+    TEST_EQUAL(db.get_doccount(), 1);
+    TEST_EQUAL(db.get_metadata("foo"), "baz");
+    db.flush();
 
-	db.add_document(doc);
-    }
+    TEST_EQUAL(dbr.get_metadata("foo"), "bar");
+    dbr.reopen();
+    TEST_EQUAL(dbr.get_metadata("foo"), "baz");
 
-    Xapian::MatchSpy spy;
+    db.set_metadata("foo", "");
+    TEST_EQUAL(db.get_metadata("foo"), "");
+    db.flush();
+    TEST_EQUAL(dbr.get_metadata("foo"), "baz");
+    dbr.reopen();
+    TEST_EQUAL(dbr.get_metadata("foo"), "");
 
-    spy.add_category(0);
-    spy.add_category(1);
-    spy.add_category(2);
-    spy.add_category(3);
-
-    Xapian::Enquire enq(db);
-
-    enq.set_query(Xapian::Query("all"));
-
-    Xapian::MSet mset = enq.get_mset(0, 10, 0, NULL, NULL, &spy);
-
-    TEST_EQUAL(spy.get_total(), 25);
-
-    static const string results[] = {
-	"|\240:1|\244:9|\246:3|\250:7|\251:1|\252:3|\254:1|",
-	string("|\200\0\0\0\0\0\0\0\0\244:8|\246\0\0\0\0\0\0\0\0\250:6|\251\0\0\0\0\0\0\0\0\253:7|\254\0\0\0\0\0\0\0\0\254\200:4|", 54),
-	string("|\243\34q\307\34q\307\0\0\271q\307\34q\307\34\300:9|\272\254q\307\34q\307@\0\301\2168\343\2168\343\300:4|\302k\2168\343\2169\0\0\304+\34q\307\34q\200:3|\304\261\307\34q\307\34@\0\305\327\34q\307\34q\200:3|\306u\307\34q\307\34@\0\307\313\34q\307\34q\200:3|\310@\343\2168\343\216\0\0\310\2408\343\2168\343\200:2|\311\3\2168\343\2168\300:1|", 132),
-	"|\237\315\36\270Q\353\205\36\300\237\321\321t]\27E\321\200:15|\237\322ffffff\200\237\325UUUUUU@:5|\237\326ffffff\200\237\330:2|\237\331UUUUUU@:1|\237\334:1|\240:1|",
-	"|",
-	""
-    };
-    for (Xapian::valueno v = 0; !results[v].empty(); ++v) {
-	bool result = spy.build_numeric_ranges(v, 7);
-	if (results[v] == "|") {
-	    TEST(!result);
-	    continue;
-	}
-	TEST(result);
-	const map<string, size_t> & cat = spy.get_categories(v);
-	TEST(cat.size() <= 7);
-	string str("|");
-	map<string, size_t>::const_iterator i;
-	for (i = cat.begin(); i != cat.end(); ++i) {
-	    str += i->first;
-	    str += ':';
-	    str += om_tostring(i->second);
-	    str += '|';
-	}
-	tout << "value " << v << endl;
-	TEST_STRINGS_EQUAL(str, results[v]);
-    }
+    TEST_EQUAL(db.get_doccount(), 1);
 
     return true;
 }
 
-// #######################################################################
-// # End of test cases: now we list the tests to run.
+// Test the empty metadata keys give an error correctly.
+DEFINE_TESTCASE(metadata3, metadata) {
+    Xapian::WritableDatabase db = get_writable_database();
 
-/// The tests which use a writable backend
-test_desc writabledb_tests[] = {
-    TESTCASE(adddoc1),
-    TESTCASE(adddoc2),
-    TESTCASE(adddoc3),
-    TESTCASE(implicitendsession1),
-    TESTCASE(databaseassign1),
-    TESTCASE(deldoc1),
-    TESTCASE(deldoc2),
-    TESTCASE(deldoc3),
-    TESTCASE(deldoc4),
-    TESTCASE(deldoc5),
-    TESTCASE(deldoc6),
-    TESTCASE(replacedoc1),
-    TESTCASE(replacedoc2),
-    TESTCASE(replacedoc3),
-    TESTCASE(replacedoc4),
-    TESTCASE(replacedoc5),
-    TESTCASE(uniqueterm1),
-    TESTCASE(emptyterm2),
-    TESTCASE(phraseorneartoand1),
-    TESTCASE(longpositionlist1),
-    TESTCASE(allpostlist2),
-    TESTCASE(consistency2),
-    TESTCASE(crashrecovery1),
-    TESTCASE(nomoredocids1),
-    TESTCASE(spell1),
-    TESTCASE(spell2),
-    TESTCASE(spell3),
-    TESTCASE(spell4),
-    TESTCASE(synonymitor1),
-    TESTCASE(matchspy1),
-    TESTCASE(matchspy2),
-    END_OF_TESTCASES
-};
+    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.get_metadata(""));
+    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.set_metadata("", "foo"));
+    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.get_metadata(""));
+
+    return true;
+}
+
+// Test that adding a document with a really long term gives an error on
+// add_document() rather than on flush().
+DEFINE_TESTCASE(termtoolong1, writable) {
+    // Quartz doesn't perform this check.
+    SKIP_TEST_FOR_BACKEND("quartz");
+    // Inmemory doesn't impose a limit.
+    SKIP_TEST_FOR_BACKEND("inmemory");
+#ifndef XAPIAN_HAS_FLINT_BACKEND
+    // If flint is disabled, remotetcp and remoteprog will use quartz
+    // which doesn't perform this check.
+    SKIP_TEST_FOR_BACKEND("remoteprog");
+    SKIP_TEST_FOR_BACKEND("remotetcp");
+#endif
+
+    Xapian::WritableDatabase db = get_writable_database();
+
+    for (Xapian::doccount i = 246; i <= 290; ++i) {
+	tout << "Term length " << i << endl;
+	Xapian::Document doc;
+	string term(i, 'X');
+	doc.add_term(term);
+	TEST_EXCEPTION(Xapian::InvalidArgumentError, db.add_document(doc));
+    }
+
+    for (Xapian::doccount j = 240; j <= 245; ++j) {
+	tout << "Term length " << j << endl;
+	Xapian::Document doc;
+	string term(j, 'X');
+	doc.add_term(term);
+	db.add_document(doc);
+    }
+
+    db.flush();
+
+    return true;
+}
+
+/// Test playing with a postlist
+DEFINE_TESTCASE(postlist7, writable) {
+    Xapian::WritableDatabase db_w = get_writable_database();
+
+    {
+	Xapian::Document doc;
+	doc.add_term("foo", 3);
+	doc.add_term("zz", 4);
+	db_w.replace_document(5, doc);
+    }
+
+    Xapian::PostingIterator p;
+    p = db_w.postlist_begin("foo");
+    TEST(p != db_w.postlist_end("foo"));
+    TEST_EQUAL(*p, 5);
+    TEST_EQUAL(p.get_wdf(), 3);
+    TEST_EQUAL(p.get_doclength(), 7);
+    ++p;
+    TEST(p == db_w.postlist_end("foo"));
+
+    {
+	Xapian::Document doc;
+	doc.add_term("foo", 1);
+	doc.add_term("zz", 1);
+	db_w.replace_document(6, doc);
+    }
+
+    p = db_w.postlist_begin("foo");
+    TEST(p != db_w.postlist_end("foo"));
+    TEST_EQUAL(*p, 5);
+    TEST_EQUAL(p.get_wdf(), 3);
+    TEST_EQUAL(p.get_doclength(), 7);
+    ++p;
+    TEST(p != db_w.postlist_end("foo"));
+    TEST_EQUAL(*p, 6);
+    TEST_EQUAL(p.get_wdf(), 1);
+    TEST_EQUAL(p.get_doclength(), 2);
+    ++p;
+    TEST(p == db_w.postlist_end("foo"));
+
+    return true;
+}
