@@ -46,6 +46,7 @@
 #include <float.h> // For DBL_DIG.
 #include <math.h> // For ceil, fabs, log10.
 #include <stdlib.h>
+#include <string.h>
 
 #include "gnu_getopt.h"
 
@@ -53,6 +54,12 @@
 #include <signal.h>
 
 #include <exception>
+#ifdef USE_RTTI
+# include <typeinfo>
+# if defined __GNUC__ && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
+#  include <cxxabi.h>
+# endif
+#endif
 
 #include <xapian/error.h>
 #include "testsuite.h"
@@ -76,7 +83,7 @@ static int vg_log_fd = -1;
 const char * expected_exception = NULL;
 
 /// The debug printing stream
-om_ostringstream tout;
+std::ostringstream tout;
 
 int test_driver::runs = 0;
 test_driver::result test_driver::total = {0, 0, 0};
@@ -258,6 +265,9 @@ test_driver::runtest(const test_desc *test)
     } \
     out << " " << col_red << M << col_reset; \
 } while (0)
+		    // Record the current position so we can restore it so
+		    // REPORT_FAIL_VG() gets the whole output.
+		    off_t curpos = lseek(vg_log_fd, 0, SEEK_CUR);
 		    char buf[1024];
 		    while (true) {
 			ssize_t c = read(vg_log_fd, buf, sizeof(buf));
@@ -266,19 +276,39 @@ test_driver::runtest(const test_desc *test)
 			    break;
 			}
 			if (c > 0) {
+			    // Valgrind output has "==<pid>== \n" between
+			    // report "records", so skip to the next occurrence
+			    // of ' ' not followed by '\n'.
 			    ssize_t i = 0;
 			    do {
-				while (i < c && buf[i] != ' ') ++i;
+				const char * spc;
+				spc = static_cast<const char *>(
+					memchr(buf + i, ' ', c - i));
+				if (!spc) {
+				    i = c;
+				    break;
+				}
+				i = spc - buf;
 			    } while (++i < c && buf[i] == '\n');
-			    char *p = reinterpret_cast<char *>(memchr(buf + i, '\n', c - i));
-			    if (p == NULL) p = buf + c;
-			    c = p - buf - i;
-			    if (c > 1024) c = 79;
-			    memmove(buf, buf + i, c);
+
+			    char *start = buf + i;
+			    c -= i;
+			    if (c > 128) c = 128;
+
+			    {
+				const char *p;
+				p = static_cast<const char*>(
+					memchr(start, '\n', c));
+				if (p != NULL) c = p - start;
+			    }
+
+			    memmove(buf, start, c);
 			    buf[c] = '\0';
 			    break;
 			}
 		    }
+		    lseek(vg_log_fd, curpos, SEEK_SET);
+
 		    VALGRIND_DO_LEAK_CHECK;
 		    int vg_errs2 = VALGRIND_COUNT_ERRORS;
 		    vg_errs = vg_errs2 - vg_errs;
@@ -384,6 +414,36 @@ test_driver::runtest(const test_desc *test)
 		    out << msg.substr(0, cutoff) << "...";
 		out << col_reset;
 		return FAIL;
+	    } catch (const std::exception & e) {
+		string s = tout.str();
+		if (!s.empty()) {
+		    out << '\n' << tout.str();
+		    if (s[s.size() - 1] != '\n') out << endl;
+		    tout.str("");
+		}
+		out << " " << col_red;
+#ifndef USE_RTTI
+		out << "std::exception";
+#else
+		const char * name = typeid(e).name();
+# if defined __GNUC__ && (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
+		// __cxa_demangle() apparently requires GCC >= 3.1.
+		// Demangle the name which GCC returns for type_info::name().
+		int status;
+		char * realname = abi::__cxa_demangle(name, NULL, 0, &status);
+		if (realname) {
+		    out << realname;
+		    free(realname);
+		} else {
+		    out << name;
+		}
+# else
+		out << name;
+# endif
+#endif
+		out << ": " << e.what();
+		out << col_reset;
+		return FAIL;
 	    } catch (...) {
 		string s = tout.str();
 		if (!s.empty()) {
@@ -658,5 +718,6 @@ test_driver::run(const test_desc *tests)
 bool
 TEST_EQUAL_DOUBLE_(double a, double b)
 {
+    if (a == b) return true;
     return (ceil(log10(max(fabs(a), fabs(b)))) - log10(fabs(a - b)) > DBL_DIG);
 }
