@@ -23,9 +23,13 @@
 #define OM_HGUARD_TESTSUITE_H
 
 #include "noreturn.h"
-#include "omstringstream.h"
+
+#include "output.h"
+
+#include "stringutils.h" // For STRINGIZE().
 
 #include <map>
+#include <sstream>
 #include <string>
 #include <vector>
 
@@ -83,7 +87,7 @@ extern const char * expected_exception;
 /** The output stream.  Data written to this stream will only appear
  *  when a test fails.
  */
-extern om_ostringstream tout;
+extern std::ostringstream tout;
 
 /// The test driver.  This class takes care of running the tests.
 class test_driver {
@@ -204,12 +208,6 @@ class test_driver {
 	static bool use_cr;
 };
 
-#ifndef STRINGIZE
-/// STRINGIZE converts a piece of code to a string, so it can be displayed.
-#define STRINGIZE(N) _STRINGIZE(N)
-#define _STRINGIZE(N) #N
-#endif
-
 /// Display the location at which a testcase occurred, with an explanation.
 #define TESTCASE_LOCN(a) __FILE__":"STRINGIZE(__LINE__)": "STRINGIZE(a)
 
@@ -246,9 +244,34 @@ extern bool TEST_EQUAL_DOUBLE_(double a, double b);
 	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' to be (nearly) equal:" \
 	" were " << setprecision(DBL_DIG) << (a) << " and " << (b) << ")" << setprecision(6))
 
+/// Test two doubles for non-near-equality.
+#define TEST_NOT_EQUAL_DOUBLE(a, b) TEST_AND_EXPLAIN(!TEST_EQUAL_DOUBLE_((a), (b)), \
+	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' not to be (nearly) equal:" \
+	" were " << setprecision(DBL_DIG) << (a) << " and " << (b) << ")" << setprecision(6))
+
 /// Test for non-equality of two things.
 #define TEST_NOT_EQUAL(a, b) TEST_AND_EXPLAIN(((a) != (b)), \
 	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' not to be equal:" \
 	" were " << (a) << " and " << (b))
 
+/// Test for one thing being >= another.
+#define TEST_GREATER_OR_EQUAL(a, b) TEST_AND_EXPLAIN(((a) >= (b)), \
+	"Expected `"STRINGIZE(a)"' to be greater than or equal to `"STRINGIZE(b)"':" \
+	" were " << (a) << " and " << (b))
+
+/// Test for one thing being > another.
+#define TEST_GREATER(a, b) TEST_AND_EXPLAIN(((a) > (b)), \
+	"Expected `"STRINGIZE(a)"' to be greater than `"STRINGIZE(b)"':" \
+	" were " << (a) << " and " << (b))
+
+/// Test for one thing being <= another.
+#define TEST_LESSER_OR_EQUAL(a, b) TEST_AND_EXPLAIN(((a) >= (b)), \
+	"Expected `"STRINGIZE(a)"' to be less than or equal to `"STRINGIZE(b)"':" \
+	" were " << (a) << " and " << (b))
+
+/// Test for one thing being < another.
+#define TEST_LESSER(a, b) TEST_AND_EXPLAIN(((a) > (b)), \
+	"Expected `"STRINGIZE(a)"' to be less than than `"STRINGIZE(b)"':" \
+	" were " << (a) << " and " << (b))
+
 #endif // OM_HGUARD_TESTSUITE_H
